/*************************************************************************
 *
 * IPAT Interferometer threshold curve plotting utility
 * v2.3 7/30/96
 *
 *************************************************************************
 * Original header:
 *
 * Copyright -C- 1981 Giles C. Billingsley
 *
 *     KIC is a graphics editor that was developed by the integrated
 * circuits group of the Electronics Research Laboratory and the
 * Department of Electrical Engineering and Computer Sciences at
 * the University of California, Berkeley, California.  The program
 * KIC is available free of charge to any interested party.
 * The sale, resale, or use of this program for profit without the
 * express written consent of the Department of Electrical Engineering
 * and Computer Sciences, University of California, Berkeley, California,
 * is forbidden.
 *
 *************************************************************************/
 
#include "mfb.h"
#include "driver.h"
#include "fb.h"
#include <ctype.h>


#ifdef __STDC__
static int  MY_MFBOpen(char*);
static int  *SetButtonMask(void);
#else
static int  MY_MFBOpen();
static int  *SetButtonMask();
#endif

struct f FB = { 0 };

#define MFBScaledText NULL


struct sGinterface GR[FBNUMINTERFACE] = {
{   MFBInfo,
    MY_MFBOpen,
    MFBClose,
    MFBUpdate,
    MFBPixel,
    MFBLine,
    MFBBox,
    MFBPolygon,
    MFBDefineLineStyle,
    MFBSetLineStyle,
    MFBDefineFillPattern,
    MFBSetFillPattern,
    MFBDefineColor,
    MFBSetColor,
    MFBText,
    MFBScaledText },

{   GR_info,
    GR_open,
    GR_close,
    GR_update,
    GR_pixel,
    GR_line,
    GR_box,
    GR_polygon,
    GR_defLs,
    GR_setLs,
    GR_defFp,
    GR_setFp,
    GR_defColor,
    GR_setColor,
    GR_text,
    GR_scText },
};


static int
MY_MFBOpen(name)

char *name;
{
    int error;

    if (MFBOpen(name,name,&error) == NULL) {
        fprintf(stderr,"%s\n",MFBError(error));
        return (1);
    }
    return (0);
}


void
FBBegin(Dspl)

char *Dspl;
{
    FB.fDisplay = Dspl;
    if (!(FB.fInitialized)) {
        if ((*GR[FB.fInterface].gOpen)(FB.fDisplay)) {
            exit(1);
        }
    }

    FBSetLineStyle(0);
    FB.fMaxX                    = FBInfo(MAXX);
    FB.fMaxY                    = FBInfo(MAXY);
    FB.fMaxP                    = 1000;
    FB.fMaxIntensity            = 255;
    FB.fNumColors               = FBInfo(MAXCOLORS) - 1;
    FB.fNumFillPatterns         = FBInfo(MAXFILLPATTERNS) - 1;
    FB.fButtonMask              = SetButtonMask();
    FB.fButtons                 = FBInfo(POINTINGBUTTONS);
    FB.fNumButtons              = FBInfo(NUMBUTTONS);
    /*
     * Font width and height is henceforth assumed to
     * include the X and Y offsets.
     */
    FB.fFontWidth               = FBInfo(FONTWIDTH);
    FB.fFontHeight              = FBInfo(FONTHEIGHT);
    FB.fFontWidth              += FBInfo(FONTXOFFSET);
    FB.fFontHeight             += FBInfo(FONTYOFFSET);
    FB.fNumRows                 = FB.fMaxY/FB.fFontHeight;
    FB.fNumColumns              = FB.fMaxX/FB.fFontWidth;
    FB.fFilledPolygons          = FBInfo(FILLEDPOLYGONS);
    FB.fDefinableFillPatterns   = FBInfo(DEFFILLPATTERNS);
    FB.fNonDestructiveText      = FBInfo(OVERSTRIKETEXT);
    FB.fLastCursorColumn        = 0;
    FB.fBlinkers                = FBInfo(BLINKERS);
    FB.fInitialized             = True;
    MFBSetTextMode(False);
    FBSetTextClip(0,0,FB.fMaxX,FB.fMaxY);
}


static int *
SetButtonMask()

{
    MFBCurrent->buttonMask[0]   = FBInfo(BUTTON1);
    MFBCurrent->buttonMask[1]   = FBInfo(BUTTON2);
    MFBCurrent->buttonMask[2]   = FBInfo(BUTTON3);
    MFBCurrent->buttonMask[3]   = FBInfo(BUTTON4);
    MFBCurrent->buttonMask[4]   = FBInfo(BUTTON5);
    MFBCurrent->buttonMask[5]   = FBInfo(BUTTON6);
    MFBCurrent->buttonMask[6]   = FBInfo(BUTTON7);
    MFBCurrent->buttonMask[7]   = FBInfo(BUTTON8);
    MFBCurrent->buttonMask[8]   = FBInfo(BUTTON9);
    MFBCurrent->buttonMask[9]   = FBInfo(BUTTON10);
    MFBCurrent->buttonMask[10]  = FBInfo(BUTTON11);
    MFBCurrent->buttonMask[11]  = FBInfo(BUTTON12);
    return (MFBCurrent->buttonMask);
}
