/**************************************************************************
 IPAT release 2.2 11/14/92

 Update 2.3 7/30/96
   * Fix bug in solving for 3-jjs
   * Add Ileft/Ig parameter
   * add coupling factor option
   * postscript hardcopy support

 Update 2.3a 9/26/96
   * fix marker printout to reflect coupling
   * fix plpts interface to reflect coupling

 Update 2.3b 8/6/97
   * fix library name clash (getopt -> get_opt)
   * fix gate current readout with non-unit coupling

 Update 2.3c 11/2/98
   * support for TrueColor visuals
   * support for .ipatrc file

 Update 2.3d 11/29/98
   * Support for TrueColor graphics when a PseudoColor visual is also
     available but is not the default (i.e., it wouldn't start under
     certain PC X-Servers).

 THIS IS FREE SOFTWARE, NO GUARANTEES, NO LIABILITY, NO NOTHING!

 Copyright (C) Stephen R. Whiteley 1992
 Please send bug reports or comments to stevew@srware.com

 This program plots the interference patterns of two and three junction
 interferometers under the following assumptions:

 1.  The coupled flux is represented by a current in the loop inductors
     with a transfer ratio of unity from a control line.

 2.  The gate current injected into the three junction interferometer
     is evenly split between the two branches.

 3.  The junctions are perfect RSJ with no coupled flux.

 IPAT compiles under Borland C (DOS real mode), Microway C (DOS
 protected mode) and UNIX compilers.  It is linked with an emulation
 library for the Berkeley MFB calls.  Under UNIX, IPAT runs under the
 MIT X window system, using an X implementation of the MFB calls.

 An argument is expected to be a file path in the format of the output
 file from jspice3 operating range analysis.  The pass or fail points
 are plotted along with the interference pattern.

 After each plot, the following arguments can be entered:

 x      Toggles whether the primary lobe (mode 0) is centered or at the
        left side of the display.

 y      Toggles whether the full Y-axis is displayed, or only the
        positive values.

 c      Enables a marker and printout of the Ic-Ig coordinates.  Retire
        the marker by hitting any key or the middle mouse button, or Enter
        if keyboard translation is enabled, (DOS only). If the
        pointer type is "keys" (as set in the mfb.rc file, DOS only),
        the marker translates with the arrow keys.  Holding the Ctrl or
        Alt buttons will slow/speed the translation.  Under UNIX, or DOS
        if "mouse" or "pad" is selected in the mfb.rc file (and a Microsoft
        mouse driver is present if "mouse" is selected) the mouse controls
        the marker position.  The left mouse button marks a location
        which is used as a reference for the coordinate printing, which
        can be deleted with the right button (i.e., go back to absolute
        coordinates).

 s      Toggles whether or not the grid is displayed.

 1-4    Scales the number of periods displayed.

 e      Prompts for a file in the format of the jspice3 operating range
        analysis to plot.

 f      Toggles whether pass or fail points are displayed.

 -      Don't show the points from the last file specified, on the
        command line or with the e option.

 p      Print the screen (DOS only, for HP Laserjet or compatible).

 h      Print a summary of the options available.

 t      Toggle display of two or three junction curves.

 r      Toggle display of tertiary contours for three junctions (left
        and right loop modes differ by 4*PI).

 n      Toggle display of secondary contours for three junctions (left
        and right loop modes differ by 2*PI).

 q      Quit (may also be entered in numerical entry field).

 z      Junction phase plot.

 The options t and q are also recognized in the input value field.


 IPAT numerically solves the equations derivable using the method of
 Tsang and Van Duzer, i.e., setting a function equal to the sum of the
 junction currents plus Lagrange multipliers multiplied by the flux
 conservation expressions for each loop.  The partial derivatives of this
 function with respect to the phases, and the Lagrange multipliers,
 yields the set of equations which are solved.

 **************************************************************************/

#include <ctype.h>
#include <math.h>
#include <ctype.h>
#include <signal.h>
#include <stdlib.h>
#include "mfb.h"
#include "fb.h"

#if defined(MSDOS) || __MSDOS__
/* path to directory which contains the mfb.rc file.  This file tells
 * the DOS MFB about the pointing device.
 */
char *MFBRCD = "c:";
#else
#define XWINDOWS
#endif

/* name of initialization file */
/* Format is:
 *  #          (comments have '#' in first column)
 *  2          (number of junctions, any text may follow number)
 *  .8         (coupling factor, any text may follow number)
 *  lpr -h     (printer command, or to file if space only)
 */
#define IPATRC ".ipatrc"

/* redraw codes */
static int Redraw, DoingRedraw, DoingPrint;
#define RD_NONE 0
#define RD_DISP 1
#define RD_INPT 2
#define RD_HELP 3

#ifndef PI
#define PI 3.1415926535898
#endif

int    SCALE = 2;        /* scale factor, 1,2,4,8 where 8 is 1 period    */
int    NITER = 8;        /* maximum number of iterations                 */
double PERIOD;           /* interference pattern period, 100% coupling   */
double COUPLING = 1.0;   /* current coupling ratio                       */
double IC;               /* maximum critical current                     */
double PHI=3.038393;     /* two pi over phi0 in normalized units         */
char   BUF[80];          /* general purpose buffer                       */
double Order1,Order2;    /* three jj phase offset parameters             */


struct {            /* screen  constants         */
        int X;      /* scale width               */
        int Y;      /* scale height              */
        int W;      /* font width                */
        int H;      /* font height               */
        int W2;     /* half font width           */
        int H2;     /* half font height          */
        int H3;     /* vertical space and a half */
        int XL;     /* left margin               */
        int XU;     /* right      "              */
        int YL;     /* lower      "              */
        int YU;     /* upper      "              */
        int XS;     /* screen width              */
        int YS;     /* screen height             */
        int XC;     /* X center of plot area     */
        int YC;     /* Y center of plot area     */
} scr;

struct {            /* parameter storage */
        double la;
        double lb;
        double lc;
        double ld;
        double ix;
        double i2;
        double iy;
        double lfrac;
        int sign;
} Vals = {.5,.5,.5,.5,1.0,1.0,1.0,.5,1};

struct extra {      /* linked list of x-y file points to plot */
        float *x;
        float *y;
        int *pf;
        int size;
        struct extra *next;
} *pts;

struct {            /* display mode flag storage */
        char c;
        char f;
        char n;
        char r;
        char s;
        char x;
        char y;
        char z;
        char threej;
        char mark;
        double rel_x;
        double rel_y;
} flags;

struct {            /* scale factors for plot */
        double xmin;
        double xmax;
        double ymin;
        double ymax;
        int    mag;
        int    xx;
        int    xo;
        int    xmn;
        int    xmx;
        int    yy;
        int    yo;
} scale;

#define sn(z) (((z) >= 0) ? 1 : 0)


#if __STDC__
extern void grafinit(void);
extern void displ(void);
extern void twojj(void);
extern void threejj(void);
extern Bool solve12(double,double*);
extern Bool solve13(double,double*);
extern Bool solve32(double,double*);
extern Bool solve31(double,double*);
extern Bool solve21(double,double*);
extern Bool solve23(double,double*);
extern Bool solcheck(double,double);
extern Bool get1(double,double,double*);
extern Bool get2(double,double,double*);
extern Bool get3(double,double,double*);
extern void showfacs(void);
extern void plotphi(double,double,double,double,double);
extern void plot(double,double);
extern Bool get_opt(void);
extern void showhelp(void);
extern void axes(void);
extern void dots(void);
extern Bool entr2(void);
extern Bool entr3(void);
extern int  getinput(double*);
extern Bool dataok(void);
extern Bool getfn(void);
extern void extpts(FILE*);
extern void plpts(void);
extern void big_pixel(int,int);
extern void delpts(void);
extern void xy_pick(void);
extern void marker(int,int,int,int);
extern void refmark();
extern Bool diag(void);
extern Bool diag2(void);
extern Bool diag3(void);
extern char *KbEdit(char*,int,int,int,int,int);
extern void KbRepaint(int,int);
extern void textcursor(int,int);
extern void clear_screen(void);
extern void show_l_icon(int,int);
extern void show_x_icon(int,int);
extern void show_s_icon(int,int);
extern void printscreen(int);
extern char *ecvt12(double);
#else
extern void grafinit();
extern void displ();
extern void twojj();
extern void threejj();
extern Bool solve12();
extern Bool solve13();
extern Bool solve32();
extern Bool solve31();
extern Bool solve21();
extern Bool solve23();
extern Bool solcheck();
extern Bool get1();
extern Bool get2();
extern Bool get3();
extern void showfacs();
extern void plotphi();
extern void plot();
extern Bool get_opt();
extern void showhelp();
extern void axes();
extern void dots();
extern Bool entr2();
extern Bool entr3();
extern int  getinput();
extern Bool dataok();
extern Bool getfn();
extern void extpts();
extern void plpts();
extern void big_pixel();
extern void delpts();
extern void xy_pick();
extern void marker();
extern void refmark();
extern Bool diag();
extern Bool diag2();
extern Bool diag3();
extern char *KbEdit();
extern void KbRepaint();
extern void textcursor();
extern void clear_screen();
extern void show_l_icon();
extern void show_x_icon();
extern void show_s_icon();
extern void printscreen();
extern char *ecvt12();
#endif


char *PRINTCMD = "lpr -h";

int
main(argc, argv)
int argc;
char **argv;
{
    int i = 1;
    FILE *fp;
    double d;
    static char buf[84];
    char *s;

#if (__TURBOC__)
    _control87(0xffff,0x003f);
#endif
    (void)signal(SIGFPE,SIG_IGN);
    scale.mag = 1;
    printf("\nIPAT 2.3d  Copyright (C) 1992 Stephen R. Whiteley\n\n");
    printf("*** Josephson interferometer threshold curve plotting program ***\n\n");

    fp = fopen(IPATRC, "r");
    if (!fp) {
        char *s = getenv("HOME");
        if (s) {
            sprintf(buf, "%s/%s", s, IPATRC);
            fp = fopen(buf, "r");
        }
    }
    if (fp) {
        while ((s = fgets(buf, 84, fp)) != NULL && *s == '#');
        if (s) {
            i = atoi(s);
            if (i != 2 && i != 3)
                i = 2;
            flags.threej = i - 2;
            if (flags.threej)
                Vals.i2 = 2;
        }
        while ((s = fgets(buf, 84, fp)) != NULL && *s == '#');
        if (s) {
            d = atof(s);
            if (d > 0.0)
                COUPLING = d;
        }
        while ((s = fgets(buf, 84, fp)) != NULL && *s == '#');
        if (s) {
            *(s + strlen(s) - 1) = '\0';  /* zap newline */
            if (isspace(*s)) {
                while (isspace(*s))
                    s++;
                if (!*s)
                    PRINTCMD = NULL;
            }
            if (*s)
                PRINTCMD = s;
        }
        fclose(fp);
    }
    else {
        printf("Enter number of junctions (2 or 3) : ");
        fflush(stdout);
        fgets(buf, 80, stdin);
        if (sscanf(buf, "%d", &i) != 1 || i != 3)
            i = 2;
        flags.threej = (char) (i - 2);
        if (flags.threej) Vals.i2 = 2;
        printf("\nPress enter for default...\n");
        printf("Enter control line coupling factor [%f]: ", COUPLING);
        fflush(stdout);
        fgets(buf, 80, stdin);
        if (sscanf(buf, "%lf", &d) == 1 && d > 0)
            COUPLING = d;
        printf("\nSpace-Enter for file \"ipat_plot.ps\" only...\n");
        printf("Enter printer command for postscript plots [%s]: ", PRINTCMD);
        fflush(stdout);
        fgets(buf, 80, stdin);
        if (strlen(buf))
            *(buf + strlen(buf) - 1) = '\0';  /* zap newline */
        s = buf;
        if (isspace(*s)) {
            while (isspace(*s))
                s++;
            if (!*s)
                PRINTCMD = NULL;
        }
        if (*s)
            PRINTCMD = s;
    }

    grafinit();
    pts = NULL;
    while (argc-- > 1) {
        if (!access(argv[i],"r")) {
            fp = fopen(argv[i++],"r");
            extpts(fp);
        }
    }

    for (;;) {
        if (!diag()) {
            for (;;) {
                displ();
                if (!get_opt()) break;
            }
        }
    }
    return (0);
}


void
grafinit()

/* initialize the graphics */
{
    int fheight, fwidth;
    char *dsp, *dd, *getenv();
    void SetWhereDisplayWindow(), SetColors();

    if (!DoingRedraw) {
        dsp = getenv("DISPLAY");
        if (dsp != NULL) {
            dd = malloc(strlen(dsp)+1);
            if (dd) {
                strcpy(dd,dsp);
                dsp = dd;
            }
        }
        SetWhereDisplayWindow(20,10,60,60);
#ifdef XWINDOWS
        NUM_XMFB_COLORS = 16;
#endif
        FBBegin(dsp);
        SetColors();
#ifdef XWINDOWS
        FBSetTextMode(0);
        FBSetName("IPAT","ipat");
#endif
    }

    fheight = FBInfo(FONTHEIGHT);
    fwidth  = FBInfo(FONTWIDTH);
    scr.W   = fwidth;
    scr.H   = fheight;
    scr.W2  = fwidth/2;
    scr.H2  = fheight/2;
    scr.H3  = scr.H + scr.H2;
    scr.XL  = 4*fwidth;
    scr.YL  = 3*fheight;
    scr.XS  = FBInfo(MAXX);
    scr.YS  = FBInfo(MAXY);
    scr.X   = scr.XS - scr.XL - 2*fwidth;
    scr.Y   = scr.YS - scr.YL - 3*fheight;
    scr.XC  = scr.XL + scr.X/2;
    scr.YC  = scr.YL + scr.Y/2;
    scr.XU  = scr.XL + scr.X;
    scr.YU  = scr.YL + scr.Y;
#ifdef XWINDOWS
    FBResizeDrawingWindow(0,0,scr.XS,scr.YS);
#endif
}


void
displ()

/* main driver for plot generation */
{
    Redraw = RD_DISP;
    PERIOD = 2.067930/(Vals.la+Vals.lb+Vals.lc+Vals.ld);
    if (flags.threej)
        PERIOD *= 2;

    scale.ymax = IC = Vals.ix + Vals.i2 + Vals.iy;

    if (flags.y) {
        scale.ymin = -scale.ymax;
        scale.yy = scr.Y/2;
        scale.yo = scr.YC;
    }
    else {
        scale.ymin = 0;
        scale.yy = scr.Y;
        scale.yo = scr.YL;
    }

    if (flags.x) {
        scale.xmax = PERIOD*4/SCALE;
        scale.xmin = -scale.xmax;
        scale.xmx = scr.X/2;
        scale.xmn = scale.xmx;
        scale.xo = scr.XC;
    }
    else {
        scale.xmax = PERIOD*8/SCALE;
        scale.xmin = 0;
        scale.xmx = scr.X;
        scale.xmn = 0;
        scale.xo = scr.XL;
    }
    scale.xx = SCALE * scr.X;

    axes();
    flags.c = 0;

    if (flags.z) {
        FBSetColor(1);
        FBText("Pi",scr.XL-2*scr.W-scr.W2,scr.YU-scr.H,0);
        FBText("-Pi",scr.XL-3*scr.W-scr.W2,scr.YL,0);
        FBText("-Pi",scr.XL,scr.YL-scr.H-1,0);
        FBText("Pi",scr.XU-2*scr.W,scr.YL-scr.H-1,0);
        FBSetColor(2);
        FBText("phi1",scr.XL+6*scr.W,scr.YL-scr.H-1,0);
        FBSetColor(3);
        FBText("phi2",scr.XL+12*scr.W,scr.YL-scr.H-1,0);
        if (flags.threej) {
            FBSetColor(4);
            FBText("phi3",scr.XL+18*scr.W,scr.YL-scr.H-1,0);
        }
        FBSetColor(1);
        FBText("arctan(Ic/Flux)",scr.XC-7*scr.W,scr.YL-scr.H-1,0);
    }

    if (!flags.threej) {  /* plot the contours for 2 junctions */

        twojj();
        return;
    }
    Order1 = Order2 = 0;
    threejj();
    Order1 = 2*PI;
    if (!flags.n) {
        /* order 1 contours */
        threejj();
    }
    Order2 = -2*PI;
    if (!flags.r) {
        /* order 2 contours */
        threejj();
    }

    showfacs();
}


void
twojj()

/* display curve for two junction interferometer */
{

    /*********************************************************************

        Basic equations:

        (1)  It = Ic1*sin(phi1) + Ic2*sin(phi2)

        (2)  FLX = phi1 - phi2 + PHI*( L1*Ic1*sin(phi1)  -
                 L2*Ic2*sin(phi2) )

        (3)  1/(Ic1*cos(phi1)) + 1/(Ic2*cos(phi2) + PHI*(L1+L2) = 0

        We loop over phi1, then phi2 (to better fill in curve), using
        (3) to get the other angle. (1) and (2) give the coordinate to plot.

    **********************************************************************/

    double phi1, phi2, flx, ic;
    double f1, f2, f3, d;
    double s1, s2;

    f1 = PHI*(Vals.la + Vals.lb);

    for (phi2 = -PI; phi2 <= PI; phi2 += .01) {

        f2 = Vals.i2*cos(phi2);
        if (f2 == 0) continue;
        f2 = 1/f2;
        f3 = Vals.ix*(f1 + f2);
        if (f3 == 0) continue;
        f3 = -1/f3;
        if (f3 > 1 || f3 < -1) continue;
        if (fabs(phi2) > .5*PI && f3 < 0) continue;
        phi1 = acos(f3);
        s1 = Vals.ix*sin(phi1);
        s2 = Vals.i2*sin(phi2);
        ic = s1 + s2;
        flx = phi1 - phi2 + PHI*(Vals.la*s1 - Vals.lb*s2);
        if (flags.z)
            plotphi(phi1,phi1,phi2,flx,ic);
        else
            plot(flx/(16*PI), ic/IC);
    }
    for (phi1 = -PI; phi1 <= PI; phi1 += .01) {

        f2 = Vals.ix*cos(phi1);
        if (f2 == 0) continue;
        f2 = 1/f2;
        f3 = Vals.i2*(f1 + f2);
        if (f3 == 0) continue;
        f3 = -1/f3;
        if (f3 > 1 || f3 < -1) continue;
        if (fabs(phi1) > .5*PI && f3 < 0) continue;
        phi2 = acos(f3);
        s1 = Vals.ix*sin(phi1);
        s2 = Vals.i2*sin(phi2);
        ic = s1 + s2;
        flx = phi1 - phi2 + PHI*(Vals.la*s1 - Vals.lb*s2);
        if (flags.z)
            plotphi(phi1,phi1,phi2,flx,ic);
        else
            plot(flx/(16*PI), ic/IC);
    }
    showfacs();
}


void
threejj()

/* display curves for three junction interferometer */
{

    /*********************************************************************

  Basic equations:
  sN = IcN*sin(phiN)
  cN = icN*cos(phiN)

  (1)  It = s1 + s2 + s3
          = Il + Ir, Il/It = C  ( C is splitting fraction )
       (gate current)

  (2)  f1 = phi1 - phi2 + PHI*(L1*s1 - L2*(C(s2 + s3) + (C-1)*s1)) + Order1
       (left loop flux)

       f2 = phi2 - phi3 + PHI*(L3*((1-C)*(s1 + s2) - C*s3) - l4*s3) + Order2
       (right loop flux)

  (3)  I~ = It + A*(f1 terms) + B*(f2 terms)
       (lagrange multipliers)

  (4)  Partial derivatives

       phi1:
       c1 +
          A * (1 + PHI*L1*c1 + (1-C)*PHI*L2*c1) +
          B * (1-C)*PHI*L3*c1 = 0

       phi2:
       c2 -
          A * (1 + C*PHI*L2*c2) +
          B * (1 + (1-C)*PHI*L3*c2) = 0

       phi3:
       c3 -
          A * C*PHI*L2*c3 -
          B * (1 + PHI*L4*c3 + C*PHI*L3*c3) = 0

  (5)  FLX = f1 + f2
       (total external flux)

  (6)  (L3+L4)*f1 - (L1+L2)*f2 = 0
       (assume the flux is coupled through the loop inductance)
       A common control line divides the flux between the two loops as
       above.

    **********************************************************************/

    double phi1,phi2,phi3;
    double start;
    Bool found1,found2;
    int count;

#define PHISTEP .0025*PI

    start = 0;
    for (;;) {
        count = 0;
        found1 = true;
        found2 = true;
        phi1 = start;
        phi2 = phi3 = 0;
        for (;;) {

            if (found1)
                found1 = solve13(phi1,&phi2);
            if (found2)
                found2 = solve12(phi1,&phi3);
            if (!found1 && !found2)
                break;
            phi1 += PHISTEP;
            if (phi1 > PI)
                break;
            count++;
        }

        found1 = true;
        found2 = true;
        phi1 = start;
        phi2 = phi3 = 0;
        for (;;) {

            if (found1)
                found1 = solve13(phi1,&phi2);
            if (found2)
                found2 = solve12(phi1,&phi3);
            if (!found1 && !found2)
                break;
            phi1 -= PHISTEP;
            if (phi1 < -PI)
                break;
            count++;
        }
        if (count > 10) break;
        if (start == 0) {
            start = PI/2;
            continue;
        }
        if (start == PI/2) {
            start = -start;
            continue;
        }
        break;
    }

    start = 0;
    for (;;) {
        count = 0;
        found1 = true;
        found2 = true;
        phi3 = start;
        phi1 = phi2 = 0;
        for (;;) {

            if (found1)
                found1 = solve31(phi3,&phi2);
            if (found2)
                found2 = solve32(phi3,&phi1);
            if (!found1 && !found2)
                break;
            phi3 += PHISTEP;
            if (phi3 > PI)
                break;
            count++;
        }

        found1 = true;
        found2 = true;
        phi3 = start;
        phi1 = phi2 = 0;
        for (;;) {

            if (found1)
                found1 = solve31(phi3,&phi2);
            if (found2)
                found2 = solve32(phi3,&phi1);
            if (!found1 && !found2)
                break;
            phi3 -= PHISTEP;
            if (phi3 < -PI)
                break;
            count++;
        }
        if (count > 10) break;
        if (start == 0) {
            start = PI/2;
            continue;
        }
        if (start == PI/2) {
            start = -start;
            continue;
        }
        break;
    }

    start = 0;
    for (;;) {
        count = 0;
        found1 = true;
        found2 = true;
        phi2 = start;
        phi1 = phi3 = 0;
        for (;;) {

            if (found1)
                found1 = solve21(phi2,&phi3);
            if (found2)
                found2 = solve23(phi2,&phi1);
            if (!found1 && !found2)
                break;
            phi2 += PHISTEP;
            if (phi2 > PI)
                break;
            count++;
        }

        found1 = true;
        found2 = true;
        phi2 = start;
        phi1 = phi3 = 0;
        for (;;) {

            if (found1)
                found1 = solve21(phi2,&phi3);
            if (found2)
                found2 = solve23(phi2,&phi1);
            if (!found1 && !found2)
                break;
            phi2 -= PHISTEP;
            if (phi2 < -PI)
                break;
            count++;
        }
        if (count > 10) break;
        if (start == 0) {
            start = PI/2;
            continue;
        }
        if (start == PI/2) {
            start = -start;
            continue;
        }
        break;
    }
}

/*
#define LIMIT(p) \
    if (p > PI) phinew = PI; \
    else if (p < -PI) phinew = -PI
*/

#define LIMIT(p) \
    if (p > PI) p -= 2*PI; \
    else if (p < -PI) p += 2*PI

Bool
solve12(phi1,phi3p)

double phi1,*phi3p;
{
    int count;
    double phi2,phi3;
    double s1,s2,s3;
    double f1,f2,f3,f4;
    double z,zz,lastz;
    double I,FLX;
    double lastphi,phinew;


    phi3 = lastphi = *phi3p - PI*1e-4;
    s1 = Vals.ix*sin(phi1);

    for (count = 0; count < NITER; count++) {

        if (get2(phi1,phi3,&phi2)) break;
    
        s2 = Vals.i2*sin(phi2);
        s3 = Vals.iy*sin(phi3);

        f1 = phi1 + PHI*(Vals.la*s1 -
            Vals.lb*((Vals.lfrac-1)*s1 + Vals.lfrac*s3)) + Order1;

        f2 = -phi3 + PHI*(Vals.lc*(-Vals.lfrac*s3 + (1-Vals.lfrac)*s1)
            - Vals.ld*s3) + Order2;

        f3 = -phi2 - Vals.lfrac*PHI*Vals.lb*s2;

        f4 = phi2 + (1-Vals.lfrac)*PHI*Vals.lc*s2;

        z = (Vals.lc + Vals.ld)*f1 - (Vals.la + Vals.lb)*f2;
        zz = (Vals.lc + Vals.ld)*f3 - (Vals.la + Vals.lb)*f4;

        if (sn(zz) != sn(z)) {
            f1 += f3;
            f2 += f4;
            z += zz;
        }
        else {
            f1 -= f3;
            f2 -= f4;
            z -= zz;
            phi2 = -phi2;
            s2 = -s2;
        }

        if (count > 0) {
            if (z != lastz) {
                phinew = phi3 - (phi3 - lastphi)*z/(z-lastz);
                LIMIT(phinew);
                lastphi = phi3;
                phi3 = phinew;
            }
            else
                count = NITER;

            if (fabs(z) < .001) {
                if (!solcheck(phi1,phi3)) break;
                I = s1 + s2 + s3;
                FLX = f1 + f2;
                if (flags.z)
                    plotphi(phi1,phi2,phi3,FLX,I);
                else
                    plot(FLX/(32*PI), I/IC);
                *phi3p = phi3;
                return (true);
            }
        }
        else {
            phi3 = *phi3p;
        }
        lastz = z;
    }
    return (false);
}

Bool
solve13(phi1,phi2p)

double phi1,*phi2p;
{
    int count;
    double phi2,phi3;
    double s1,s2,s3;
    double f1,f2,f3,f4;
    double z,zz,lastz;
    double I,FLX;
    double lastphi,phinew;

    phi2 = lastphi = *phi2p - PI*1e-4;
    s1 = Vals.ix*sin(phi1);

    for (count = 0; count <= NITER; count++) {

        if (get3(phi1,phi2,&phi3)) break;

        s2 = Vals.i2*sin(phi2);
        s3 = Vals.iy*sin(phi3);

        f1 = phi1 - phi2 + PHI*(Vals.la*s1 -
            Vals.lb*((Vals.lfrac-1)*s1 + Vals.lfrac*s2)) + Order1;

        f2 = phi2 + PHI*((1-Vals.lfrac)*Vals.lc*(s2 + s1)) + Order2;

        f3 = -Vals.lfrac*PHI*Vals.lb*s3;

        f4 = -phi3 -  PHI*(Vals.ld + Vals.lfrac*Vals.lc)*s3;

        z = (Vals.lc + Vals.ld)*f1 - (Vals.la + Vals.lb)*f2;
        zz = (Vals.lc + Vals.ld)*f3 - (Vals.la + Vals.lb)*f4;

        if (sn(zz) != sn(z)) {
            f1 += f3;
            f2 += f4;
            z += zz;
        }
        else {
            f1 -= f3;
            f2 -= f4;
            z -= zz;
            phi3 = -phi3;
            s3 = -s3;
        }

        if (count > 0) {
            if (z != lastz) {
                phinew = phi2 - (phi2 - lastphi)*z/(z-lastz);
                LIMIT(phinew);
                lastphi = phi2;
                phi2 = phinew;
            }
            else
                count = NITER;

            if (fabs(z) < .001) {
                if (!solcheck(phi1,phi3)) break;
                I = s1 + s2 + s3;
                FLX = f1 + f2;
                if (flags.z)
                    plotphi(phi1,phi2,phi3,FLX,I);
                else
                    plot(FLX/(32*PI), I/IC);
                *phi2p = phi2;
                return (true);
            }
        }
        else {
            phi2 = *phi2p;
        }
        lastz = z;
    }
    return (false);
}


Bool
solve32(phi3,phi1p)

double phi3,*phi1p;
{
    int count;
    double phi2,phi1;
    double s1,s2,s3;
    double f1,f2,f3,f4;
    double z,zz,lastz;
    double I,FLX;
    double lastphi,phinew;


    phi1 = lastphi = *phi1p - PI*1e-4;
    s3 = Vals.iy*sin(phi3);

    for (count = 0; count < NITER; count++) {

        if (get2(phi1,phi3,&phi2)) break;
    
        s1 = Vals.ix*sin(phi1);
        s2 = Vals.i2*sin(phi2);

        f1 = phi1 + PHI*(Vals.la*s1 -
            Vals.lb*((Vals.lfrac-1)*s1 + Vals.lfrac*s3)) + Order1;

        f2 = -phi3 + PHI*(Vals.lc*(-Vals.lfrac*s3 + (1-Vals.lfrac)*s1)
            - Vals.ld*s3) + Order2;

        f3 = -phi2 - Vals.lfrac*PHI*Vals.lb*s2;

        f4 = phi2 + (1-Vals.lfrac)*PHI*Vals.lc*s2;

        z = (Vals.lc + Vals.ld)*f1 - (Vals.la + Vals.lb)*f2;
        zz = (Vals.lc + Vals.ld)*f3 - (Vals.la + Vals.lb)*f4;

        if (sn(zz) != sn(z)) {
            f1 += f3;
            f2 += f4;
            z += zz;
        }
        else {
            f1 -= f3;
            f2 -= f4;
            z -= zz;
            phi2 = -phi2;
            s2 = -s2;
        }

        if (count > 0) {
            if (z != lastz) {
                phinew = phi1 - (phi1 - lastphi)*z/(z-lastz);
                LIMIT(phinew);
                lastphi = phi1;
                phi1 = phinew;
            }
            else
                count = NITER;

            if (fabs(z) < .001) {
                if (!solcheck(phi1,phi3)) break;
                I = s1 + s2 + s3;
                FLX = f1 + f2;
                if (flags.z)
                    plotphi(phi1,phi2,phi3,FLX,I);
                else
                    plot(FLX/(32*PI), I/IC);
                *phi1p = phi1;
                return (true);
            }
        }
        else {
            phi1 += *phi1p;
        }
        lastz = z;
    }
    return (false);
}


Bool
solve31(phi3,phi2p)

double phi3,*phi2p;
{
    int count;
    double phi1,phi2;
    double s1,s2,s3;
    double f1,f2,f3,f4;
    double z,zz,lastz;
    double I,FLX;
    double lastphi,phinew;


    phi2 = lastphi = *phi2p - PI*1e-4;
    s3 = Vals.iy*sin(phi3);

    for (count = 0; count <= NITER; count++) {

        if (get1(phi2,phi3,&phi1)) break;
    
        s1 = Vals.ix*sin(phi1);
        s2 = Vals.i2*sin(phi2);

        f1 = -phi2 - Vals.lfrac*PHI*Vals.lb*(s2 + s3) + Order1;

        f2 = phi2 - phi3 + PHI*(Vals.lc*(-Vals.lfrac*s3 + (1-Vals.lfrac)*s2)
            - Vals.ld*s3) + Order2;

        f3 = phi1 + PHI*(Vals.la + (1-Vals.lfrac)*Vals.lb)*s1;

        f4 = (1-Vals.lfrac)*PHI*Vals.lc*s1;

        z = (Vals.lc + Vals.ld)*f1 - (Vals.la + Vals.lb)*f2;
        zz = (Vals.lc + Vals.ld)*f3 - (Vals.la + Vals.lb)*f4;

        if (sn(zz) != sn(z)) {
            f1 += f3;
            f2 += f4;
            z += zz;
        }
        else {
            f1 -= f3;
            f2 -= f4;
            z -= zz;
            phi1 = -phi1;
            s1 = -s1;
        }

        if (count > 0) {
            if (z != lastz) {
                phinew = phi2 - (phi2 - lastphi)*z/(z-lastz);
                LIMIT(phinew);
                lastphi = phi2;
                phi2 = phinew;
            }
            else
                count = NITER;

            if (fabs(z) < .001) {
                if (!solcheck(phi1,phi3)) break;
                I = s1 + s2 + s3;
                FLX = f1 + f2;
                if (flags.z)
                    plotphi(phi1,phi2,phi3,FLX,I);
                else
                    plot(FLX/(32*PI), I/IC);
                *phi2p = phi2;
                return (true);
            }
        }
        else {
            phi2 = *phi2p;
        }
        lastz = z;
    }
    return (false);
}


Bool
solve21(phi2,phi3p)

double phi2,*phi3p;
{
    int count;
    double phi1,phi3;
    double s1,s2,s3;
    double f1,f2,f3,f4;
    double z,zz,lastz;
    double I,FLX;
    double lastphi,phinew;


    phi3 = lastphi = *phi3p - PI*1e-4;
    s2 = Vals.i2*sin(phi2);

    for (count = 0; count <= NITER; count++) {

        if (get1(phi2,phi3,&phi1)) break;
    
        s1 = Vals.ix*sin(phi1);
        s3 = Vals.iy*sin(phi3);

        f1 = -phi2 - Vals.lfrac*PHI*Vals.lb*(s2 + s3) + Order1;

        f2 = phi2 - phi3 + PHI*(Vals.lc*(-Vals.lfrac*s3 + (1-Vals.lfrac)*s2)
            - Vals.ld*s3) + Order2;

        f3 = phi1 + PHI*(Vals.la + (1-Vals.lfrac)*Vals.lb)*s1;

        f4 = (1-Vals.lfrac)*PHI*Vals.lc*s1;

        z = (Vals.lc + Vals.ld)*f1 - (Vals.la + Vals.lb)*f2;
        zz = (Vals.lc + Vals.ld)*f3 - (Vals.la + Vals.lb)*f4;

        if (sn(zz) != sn(z)) {
            f1 += f3;
            f2 += f4;
            z += zz;
        }
        else {
            f1 -= f3;
            f2 -= f4;
            z -= zz;
            phi1 = -phi1;
            s1 = -s1;
        }

        if (count > 0) {
            if (z != lastz) {
                phinew = phi3 - (phi3 - lastphi)*z/(z-lastz);
                LIMIT(phinew);
                lastphi = phi3;
                phi3 = phinew;
            }
            else
                count = NITER;

            if (fabs(z) < .001) {
                if (!solcheck(phi1,phi3)) break;
                I = s1 + s2 + s3;
                FLX = f1 + f2;
                if (flags.z)
                    plotphi(phi1,phi2,phi3,FLX,I);
                else
                    plot(FLX/(32*PI), I/IC);
                *phi3p = phi3;
                return (true);
            }
        }
        else {
            phi3 = *phi3p;
        }
        lastz = z;
    }
    return (false);
}


Bool
solve23(phi2,phi1p)

double phi2,*phi1p;
{
    int count;
    double phi1,phi3;
    double s1,s2,s3;
    double f1,f2,f3,f4;
    double z,zz,lastz;
    double I,FLX;
    double lastphi,phinew;

    phi1 = lastphi = *phi1p - PI*1e-4;
    s2 = Vals.i2*sin(phi2);

    for (count = 0; count <= NITER; count++) {

        if (get3(phi1,phi2,&phi3)) break;

        s1 = Vals.ix*sin(phi1);
        s3 = Vals.iy*sin(phi3);

        f1 = phi1 - phi2 + PHI*(Vals.la*s1 -
            Vals.lb*((Vals.lfrac-1)*s1 + Vals.lfrac*s2)) + Order1;

        f2 = phi2 + PHI*((1-Vals.lfrac)*Vals.lc*(s2 + s1)) + Order2;

        f3 = -Vals.lfrac*PHI*Vals.lb*s3;

        f4 = -phi3 -  PHI*(Vals.ld + Vals.lfrac*Vals.lc)*s3;

        z = (Vals.lc + Vals.ld)*f1 - (Vals.la + Vals.lb)*f2;
        zz = (Vals.lc + Vals.ld)*f3 - (Vals.la + Vals.lb)*f4;

        if (sn(zz) != sn(z)) {
            f1 += f3;
            f2 += f4;
            z += zz;
        }
        else {
            f1 -= f3;
            f2 -= f4;
            z -= zz;
            phi3 = -phi3;
            s3 = -s3;
        }

        if (count > 0) {
            if (z != lastz) {
                phinew = phi1 - (phi1 - lastphi)*z/(z-lastz);
                LIMIT(phinew);
                lastphi = phi1;
                phi1 = phinew;
            }
            else
                count = NITER;

            if (fabs(z) < .001) {
                if (!solcheck(phi1,phi3)) break;
                I = s1 + s2 + s3;
                FLX = f1 + f2;
                if (flags.z)
                    plotphi(phi1,phi2,phi3,FLX,I);
                else
                    plot(FLX/(32*PI), I/IC);
                *phi1p = phi1;
                return (true);
            }
        }
        else {
            phi1 = *phi1p;
        }
        lastz = z;
    }
    return (false);
}


Bool
solcheck(phi1,phi3)

double phi1,phi3;
{
    double co;

    co = PHI*Vals.ix*cos(phi1)*(Vals.la+Vals.lb);
    if (co < -1) return (false);
    co = PHI*Vals.iy*cos(phi3)*(Vals.lc+Vals.ld);
    if (co < -1) return (false);
    return (true);
}


Bool
get1(phi2,phi3,phi1)

double phi3,phi2,*phi1;
{
    double z1,z2,z3,z31,z32,z21,z22,a,b,c,d,e;

    z2 = Vals.i2*cos(phi2);
    z3 = Vals.iy*cos(phi3);

    z31 = -PHI*Vals.lfrac*Vals.lb*z3;
    z32 = -1 - PHI*(Vals.ld + Vals.lfrac*Vals.lc)*z3;

    z21 = -1 - PHI*Vals.lfrac*Vals.lb*z2;
    z22 = 1 + PHI*(1-Vals.lfrac)*Vals.lc*z2;

    /* z3 + A*z31 + B*z32 = 0
     * z2 + A*z21 + B*z22 = 0
     * B = -(z2 + A*z21)/z22
     * z3 + A*z31 - z32*(z2 + A*z21)/z22 = 0
     * z3 - z32*z2/z22 = A*(-z31 + z32*z21/z22)
     * A = (z3*z22 - z32*z2)/(-z31*z22 + z32*z21)
     */

    a = (z3*z22 - z32*z2);
    b = (-z31*z22 + z32*z21);
    c = -(b*z2 + a*z21);
    d = b*z22;

    /* z1 + a*(1 + PHI*L1*z1 + (1-C)*PHI*L2*z1)/b + c*((1-C)*PHI*L3*z1)/d= 0
     * z1(1 + a*PHI*(L1 + (1-C)*L2)/b + (1-C)*c*PHI*L3/d = -a/b
     * z1 = -a/(b + a*PHI*(L1 + (1-C)*L2) + (1-C)*c*PHI*L3/d)
     * z1 = -a*d/(b*d + a*d*PHI*(L1 + (1-C)*L2) + (1-C)*c*b*PHI*L3)
     */

    e = b*d + a*d*PHI*(Vals.la + (1-Vals.lfrac)*Vals.lb) +
        c*b*PHI*(1-Vals.lfrac)*Vals.lc;
    z1 = -a*d;

    if (e == 0) {
        if (z1 == 0) {
            *phi1 = .5*PI;
            return (false);
        }
        return (true);
    }

    z1  /= e*Vals.ix;

    if (fabs(z1) <= 1) {
        *phi1 = acos(z1);
        return (false);
    }
    return (true);
}


Bool
get2(phi1,phi3,phi2)

double phi1,phi3,*phi2;
{
    double z1,z2,z3,z31,z32,z11,z12,a,b,c,d,e;

    z1 = Vals.ix*cos(phi1);
    z3 = Vals.iy*cos(phi3);

    z11 = 1 + PHI*(Vals.la + (1-Vals.lfrac)*Vals.lb)*z1;
    z12 = PHI*(1-Vals.lfrac)*Vals.lc*z1;

    z31 = -PHI*Vals.lfrac*Vals.lb*z3;
    z32 = -1 - PHI*(Vals.ld + Vals.lfrac*Vals.lc)*z3;

    /* z1 + A*z11 + B*z12 = 0
     * z3 + A*z31 + B*z32 = 0
     * B = -(z3 + A*z31)/z32
     * z1 + A*z11 - z12*(z3 + A*z31)/z32 = 0
     * z1 - z12*z3/z32 = A*(-z11 + z12*z31/z32)
     * A = (z1*z32 - z12*z3)/(-z11*z32 + z12*z31);
     */
    a = (z1*z32 - z12*z3);
    b = (-z11*z32 + z12*z31);
    c = -(b*z3 + a*z31);
    d = b*z32;

    /* z2 - a*(1 + C*PHI*L2*z2)/b + c*(1 + (1-C)*PHI*L3*z2)/d = 0
     * z2(1 - C*a*PHI*L2/b + (1-C)*c*PHI*L3/d) = a/b - c/d
     * z2 = (a*d - c*b)/(b*d - C*a*d*PHI*L2 + (1-C)*c*b*PHI*L3)
     */

    e = b*d - a*d*PHI*Vals.lfrac*Vals.lb + c*b*PHI*(1-Vals.lfrac)*Vals.lc;
    z2 = a*d - c*b;

    if (e == 0) {
        if (z3 == 0) {
            *phi2 = .5*PI;
            return (false);
        }
        return (true);
    }

    z2 /= e*Vals.i2;

    if (fabs(z2) <= 1) {
        *phi2 = acos(z2);
        return (false);
    }
    return (true);
}


Bool
get3(phi1,phi2,phi3)

double phi1,phi2,*phi3;
{
    double z1,z2,z3,z11,z12,z21,z22,a,b,c,d,e;

    z1 = Vals.ix*cos(phi1);
    z2 = Vals.i2*cos(phi2);

    z11 = 1 + PHI*(Vals.la + (1-Vals.lfrac)*Vals.lb)*z1;
    z12 = PHI*(1-Vals.lfrac)*Vals.lc*z1;

    z21 = -1 - PHI*Vals.lfrac*Vals.lb*z2;
    z22 = 1 + PHI*(1-Vals.lfrac)*Vals.lc*z2;

    /* z1 + A*z11 + B*z12 = 0
     * z2 + A*z21 + B*z22 = 0
     * B = -(z2 + A*z21)/z22
     * z1 + A*z11 - z12*(z2 + A*z21)/z22 = 0
     * z1 - z12*z2/z22 = A*(-z11 + z12*z21/z22)
     * A = (z1*z22 - z12*z2)/(-z11*z22 + z12*z21);
     */
    a = (z1*z22 - z12*z2);
    b = (-z11*z22 + z12*z21);
    c = -(b*z2 + a*z21);
    d = b*z22;

    /* z3 - a*(C*PHI*L2*z3)/b - c*(1 + PHI*L4*z3 + C*PHI*L3*z3)/d = 0
     * z3(1 - C*a*PHI*L2/b - c*PHI*(L4 + C*L3)/d) = c/d
     * z3 = c/(d - C*a*PHI*L2*d/b - c*PHI*(L4 + C*L3))
     * z3 = c*b/(b*d - C*a*PHI*L2*d - c*b*PHI*(L4 + C*L3))
     */

    e = b*d - a*d*PHI*Vals.lfrac*Vals.lb - c*b*PHI*(Vals.ld +
        Vals.lfrac*Vals.lc);
    z3 = c*b;

    if (e == 0) {
        if (z3 == 0) {
            *phi3 = .5*PI;
            return (false);
        }
        return (true);
    }

    z3 /= e*Vals.iy;

    if (fabs(z3) <= 1) {
        *phi3 = acos(z3);
        return (false);
    }
    return (true);
}


void
showfacs()

/* display the parameters used in the plot */
{
    int y = scr.YU + scr.H2;
    static first;

    if (!flags.s) dots();
    FBSetColor(8);
    if (!flags.z) {
        sprintf(BUF,"Period %g mA, Coupling %g",PERIOD/COUPLING,
            COUPLING);
        FBText(BUF,scr.XL,scr.YL - scr.H3,0);
    }
    if (flags.threej) {
        sprintf(BUF,"L1  = %g", Vals.la);
        FBText(BUF,scr.XL,y+scr.H,0);
        sprintf(BUF,"L2  = %g", Vals.lb);
        FBText(BUF,scr.XL,y,0);
        sprintf(BUF,"L3  = %g", Vals.lc);
        FBText(BUF,scr.XL+scr.X/4,y+scr.H,0);
        sprintf(BUF,"L4  = %g", Vals.ld);
        FBText(BUF,scr.XL+scr.X/4,y,0);
        sprintf(BUF,"Ic1 = %g", Vals.ix);
        FBText(BUF,scr.XC,y+scr.H,0);
        sprintf(BUF,"Ic2 = %g", Vals.i2);
        FBText(BUF,scr.XC,y,0);
        sprintf(BUF,"Ic3 = %g", Vals.iy);
        FBText(BUF,scr.XC+scr.X/4,y+scr.H,0);
        sprintf(BUF,"Ileft/Ig = %g", Vals.lfrac);
        FBText(BUF,scr.XC+scr.X/4,y,0);
        FBSetColor(2);
        FBText("3 JUNCTION INTERFEROMETER",scr.XL,scr.YL-scr.H3-scr.H,0);
    }
    else {
        sprintf(BUF,"L1  = %g", Vals.la);
        FBText(BUF,scr.XL,y,0);
        sprintf(BUF,"L2  = %g", Vals.lb);
        FBText(BUF,scr.XL+scr.X/4,y,0);
        sprintf(BUF,"Ic1 = %g", Vals.ix);
        FBText(BUF,scr.XC,y,0);
        sprintf(BUF,"Ic2 = %g", Vals.i2);
        FBText(BUF,scr.XC+scr.X/4,y,0);
        FBSetColor(2);
        FBText("2 JUNCTION INTERFEROMETER",scr.XC-12*scr.W,y+scr.H,0);
    }
    plpts();
    if (!first) {
        FBSetColor(1);
        FBText("Enter option or h for help",
            scr.XU-26*scr.W,scr.YL-scr.H3,0);
        first = 1;
    }
    else show_s_icon(scr.XU-50-5*scr.W,scr.YL-32);
}


void
plot(flx,iy)

/* plot points */
double flx, iy;
{
    int x, y, n;

    n = - (int) ((flx+1)/.125);
    switch(flags.c) {
        case 0:
            FBSetColor(2+abs(n));
            break;
        case 1:
            FBSetColor(13);
            break;
        case 2:
        default:
            FBSetColor(15);
            break;
    }
    flx += n * .125;
    y = (int) (iy*scale.yy + .5);
    while (flx <= 1) { /* clip and plot points */
        x = (int) (flx*scale.xx + .5);
        if (x >= -scale.xmn && x <= scale.xmx && (flags.y || y >= 0))
            FBPixel(scale.xo + x, scale.yo + y);
        if (x <  scale.xmn && x >= -scale.xmx && (flags.y || y < 0))
            FBPixel(scale.xo - x, scale.yo - y);
        flx += .125;
        n++;
        if (!flags.c) FBSetColor(2+abs(n));
    }
}


void
plotphi(phi1,phi2,phi3,flx,i)
double phi1,phi2,phi3,flx,i;
{
    int y1,y2,y3,x;

    if (flags.threej && (Order1 != 0 || Order2 != 0))
        return;

    if (i == 0 && flx == 0)
        return;
    i = atan2(flx,i);
    x  = scr.XL + scr.X*(i + PI)/(2*PI);
    y1 = scr.YL + scr.Y*(phi1 + PI)/(2*PI);
    y2 = scr.YL + scr.Y*(phi2 + PI)/(2*PI);
    y3 = scr.YL + scr.Y*(phi3 + PI)/(2*PI);
    FBSetColor(2);
    FBPixel(x,y1);
    FBSetColor(3);
    if (flags.threej) {
        FBPixel(x,y2);
        FBSetColor(4);
    }
    FBPixel(x,y3);

    if (i >= 0)
        i -= PI;
    else
        i += PI;
    phi1 = -phi1;
    phi2 = -phi2;
    phi3 = -phi3;
    x  = scr.XL + scr.X*(i + PI)/(2*PI);
    y1 = scr.YL + scr.Y*(phi1 + PI)/(2*PI);
    y2 = scr.YL + scr.Y*(phi2 + PI)/(2*PI);
    y3 = scr.YL + scr.Y*(phi3 + PI)/(2*PI);
    FBSetColor(2);
    FBPixel(x,y1);
    FBSetColor(3);
    if (flags.threej) {
        FBPixel(x,y2);
        FBSetColor(4);
    }
    FBPixel(x,y3);
}


Bool
get_opt()

/* get and process options */    
{
    int i, c;

    while ((c = MFBGetchar()) > ' ') {
        if (isdigit(c)) {
            i = c - '0';
            if (i < 1 || i > 4) return (false);
            SCALE = 1 << (i-1);
            return (true);
        }
        switch(c) {
        case 'c': /* bring up marker */
            if (Redraw != RD_HELP)
                xy_pick();
            continue;
        case 'e': /* prompt for points file name */
            if (getfn()) return (true);
            continue;
        case 'f': /* plot fail points from file */
            flags.f ^= 1;
            return (true);
        case 'h':
            showhelp();
            continue;
        case 'n': /* toggle display of secondary lobes */
            flags.n = flags.n ? 0 : 1;
            return (true);
        case 'p':
        case 'P':
            printscreen(false);
            return (true);
        case 'q': /* die gracefully */
            FBEnd();
            exit(0);
        case 'r': /* toggle display of tertiary lobes */
            flags.r ^= 1;
            return (true);
        case 's': /* toggle display of internal axes */
            flags.s ^= 1;
            return (true);
        case 't': /* toggle 2 and 3 junction curves */
            flags.threej ^= 1;
            if (flags.threej) {
                Vals.la = Vals.lb = Vals.lc = Vals.ld = .5;
                Vals.ix = Vals.iy = 1;
                Vals.i2 = 2;
            }
            else {
                Vals.la = Vals.lb = .5;
                Vals.ix = Vals.i2 = 1;
            }
            return (false);
        case 'x': /* toggle 0 to X and -X to X display */
            flags.x ^= 1;
            return (true);
        case 'y': /* toggle 0 to Y and -Y to Y display */
            flags.y ^= 1;
            return (true);
        case 'z': /* show junction phase plot */
            flags.z ^= 1;
            return (true);
        }
    }
    return (false);
}


void
showhelp()

/* print the help screen */
{
    int x, y;

    Redraw = RD_HELP;
    clear_screen();
    x = scr.XL+7*scr.W;
    y = scr.YU-2*scr.H;
    FBSetColor(8);
    FBText("Options are the following:",x,y,0);
    FBText("c     Bring up the marker.  ENTER to retire marker.",
        x,y-=2*scr.H,0);
    FBText("      If keyboard translation, move marker with arrow",
        x,y-=scr.H,0);
    FBText("      keys - Ctrl, Alt change rate (DOS only).",
        x,y-=scr.H,0);
    FBText("      Otherwise marker moves with pointing device.",
        x,y-=scr.H,0);
    FBText("e     Prompt for file name to read points to be plotted.",
        x,y-=scr.H,0);
    FBText("f     Plot fail points from file.",
        x,y-=scr.H,0);
    FBText("h     Print this message.",x,y-=scr.H,0);
    FBText("n     Don't plot the secondary contours for 3 junctions.",
        x,y-=scr.H,0);
    FBText("p     Print the screen, leave plotfile \"hpplot\".",
        x,y-=scr.H,0);
    FBText("q     Exit this program.",x,y-=scr.H,0);
    FBText("r     Don't plot the tertiary contours for 3 junctions.",
        x,y-=scr.H,0);
    FBText("s     Don't display internal axes.",
        x,y-=scr.H,0);
    FBText("t     Two junctions or three junctions.",
        x,y-=scr.H,0);
    FBText("x     Lobe pattern centered or left axis.",
        x,y-=scr.H,0);
    FBText("y     Display negative points of y axis.",
        x,y-=scr.H,0);
    FBText("z     Plot junction phase (central lobe for 3 jj's).",
        x,y-=scr.H,0);
    FBText("1-4   Rescale the x axis.",
        x,y-=scr.H,0);
    FBText("Options f,n,r,s,t,x,y,z toggle.",x, y-=2*scr.H,0);
    FBSetColor(1);
    FBText("Enter option or ENTER to continue.",x,y-=2*scr.H,0);

    FBSetColor(2);
    FBLine(x-scr.W,y-scr.H2,x+60*scr.W,y-scr.H2);
    FBLine(x+60*scr.W,y-scr.H2,x+60*scr.W,scr.YU-scr.H2);
    FBLine(x+60*scr.W,scr.YU-scr.H2,x-scr.W,scr.YU-scr.H2);
    FBLine(x-scr.W,scr.YU-scr.H2,x-scr.W,y-scr.H2);
}


void
axes()

/* draw axes */
{

    clear_screen();
    FBSetColor(8);
    FBLine(scr.XL,scr.YL,scr.XU,scr.YL);
    FBLine(scr.XU,scr.YL,scr.XU,scr.YU);
    FBLine(scr.XU,scr.YU,scr.XL,scr.YU);
    FBLine(scr.XL,scr.YU,scr.XL,scr.YL);
    if (flags.s) return;
    FBDefineLineStyle(1,0xcc);
    FBLine(scr.XC,scr.YL,scr.XC,scr.YU);
    FBLine(scr.XL,scr.YC,scr.XU,scr.YC);
    FBSetLineStyle(0);
}


void
dots()

/* draw dots - part of axes */
{
    int i, j, dx = scr.X/8, dy = scr.Y/8;

    FBSetColor(1);
    for (i=1; i<4; i++)
        for (j=1; j<4; j++) {
            FBPixel(scr.XL+i*dx,scr.YL+j*dy);
            FBPixel(scr.XC+i*dx,scr.YL+j*dy);
            FBPixel(scr.XL+i*dx,scr.YC+j*dy);
            FBPixel(scr.XC+i*dx,scr.YC+j*dy);
        }
}

/* remember for screen redraw (used for X only) */
static int GetinpX, GetinpY, GetinpI;


Bool
entr2()

/* get parameter values - 2 junctions */
{
    int dy = scr.H, i;

    Vals.lc = 0;
    Vals.ld = 0;
    Vals.iy = 0;
    FBSetColor(5);
    sprintf(BUF,"L1  = %g", Vals.la);
    FBText(BUF,scr.XL,8*dy,0);
    sprintf(BUF,"L2  = %g", Vals.lb);
    FBText(BUF,scr.XL,6*dy,0);
    sprintf(BUF,"Ic1 = %g", Vals.ix);
    FBText(BUF,scr.XL,4*dy,0);
    sprintf(BUF,"Ic2 = %g", Vals.i2);
    FBText(BUF,scr.XL,2*dy,0);
    GetinpX = scr.XC;

    if (DoingRedraw) {
        /* redraw the colored box and return */
        i = 8 - 2*GetinpI;
        GetinpY = i*dy;
        FBSetColor(2);
        FBBox(GetinpX-scr.W-scr.W2,GetinpY+3,GetinpX-scr.W2,GetinpY+scr.H-3);
        FBSetColor(5);
        return (false);
    }
    if (DoingPrint)
        return (false);

    GetinpI = 0;

    while (GetinpI >= 0 && GetinpI < 4) {
        switch (GetinpI) {
        case 0:
            GetinpY = 8*dy;
            GetinpI += getinput(&Vals.la);
            continue;
        case 1:
            GetinpY = 6*dy;
            GetinpI += getinput(&Vals.lb);
            continue;
        case 2:
            GetinpY = 4*dy;
            GetinpI += getinput(&Vals.ix);
            continue;
        case 3:
            GetinpY = 2*dy;
            GetinpI += getinput(&Vals.i2);
            continue;
        }
    }
    if (GetinpI > 10) return (true);
    return (false);
}


Bool
entr3()

/* get parameter values - 3 junctions */
{
    int dy = scr.H, i;

    FBSetColor(5);
    sprintf(BUF,"L1  = %g", Vals.la);
    FBText(BUF,scr.XL,8*dy,0);
    sprintf(BUF,"L2  = %g", Vals.lb);
    FBText(BUF,scr.XL,7*dy,0);
    sprintf(BUF,"L3  = %g", Vals.lc);
    FBText(BUF,scr.XL,6*dy,0);
    sprintf(BUF,"L4  = %g", Vals.ld);
    FBText(BUF,scr.XL,5*dy,0);
    sprintf(BUF,"Ileft/Ig = %g", Vals.lfrac);
    FBText(BUF,scr.XL,4*dy,0);
    sprintf(BUF,"Ic1 = %g", Vals.ix);
    FBText(BUF,scr.XL,3*dy,0);
    sprintf(BUF,"Ic2 = %g", Vals.i2);
    FBText(BUF,scr.XL,2*dy,0);
    sprintf(BUF,"Ic3 = %g", Vals.iy);
    FBText(BUF,scr.XL,dy,0);
    GetinpX = scr.XC;

    if (DoingRedraw) {
        /* redraw the colored box and return */
        i = 8 - GetinpI;
        if (i >= 4) i--;
        GetinpY = i*dy;
        FBSetColor(2);
        FBBox(GetinpX-scr.W-scr.W2,GetinpY+3,GetinpX-scr.W2,GetinpY+scr.H-3);
        FBSetColor(5);
        return (false);
    }
    if (DoingPrint)
        return (false);

    GetinpI = 0;

    while (GetinpI >= 0 && GetinpI < 8) {
        switch (GetinpI) {
        case 0:
            GetinpY = 8*dy;
            GetinpI += getinput(&Vals.la);
            continue;
        case 1:
            GetinpY = 7*dy;
            GetinpI += getinput(&Vals.lb);
            continue;
        case 2:
            GetinpY = 6*dy;
            GetinpI += getinput(&Vals.lc);
            continue;
        case 3:
            GetinpY = 5*dy;
            GetinpI += getinput(&Vals.ld);
            continue;
        case 4:
            GetinpY = 4*dy;
            GetinpI += getinput(&Vals.lfrac);
            continue;
        case 5:
            GetinpY = 3*dy;
            GetinpI += getinput(&Vals.ix);
            continue;
        case 6:
            GetinpY = 2*dy;
            GetinpI += getinput(&Vals.i2);
            continue;
        case 7:
            GetinpY = dy;
            GetinpI += getinput(&Vals.iy);
            continue;
        }
    }
    if (GetinpI > 10) return (true);

    Vals.sign = 1;
    return (false);
}


int
getinput(d)

/* get double from stdin */
double *d;
{
    int inkey;
    char c, s[2];
    int i = 0, up = 1;
    double atof();

    FBSetColor(2);
    FBBox(GetinpX-scr.W-scr.W2,GetinpY+3,GetinpX-scr.W2,GetinpY+scr.H-3);
    FBSetColor(5);
    while ((c = (char) (inkey = MFBGetchar())) != 0) {
        if (c == '\b') {
            if (i) i--;
            sprintf(s,"%c",BUF[i]);
            FBSetColor(0);
            FBText(s,GetinpX + i*scr.W,GetinpY,0);
            FBSetColor(5);
            continue;
        }
        if (c < ' ') break;
        if (c == 'q') {
            FBEnd();
            exit(0);
        }
        if (c == 't') {
            flags.threej = flags.threej ? 0 : 1;
            if (flags.threej) {
                Vals.la = Vals.lb = Vals.lc = Vals.ld = .5;
                Vals.ix = Vals.iy = 1;
                Vals.i2 = 2;
            }
            else {
                Vals.la = Vals.lb = .5;
                Vals.ix = Vals.i2 = 1;
            }
            return (100);
        }
        if (c == 'p') {
            printscreen(true);
            continue;
        }
        if ((c == 'e') || (c == 'E') || (c == '.') || (c == '-')
            || isdigit(c)) {
                if ((c == '-') && (!i || (BUF[i-1] != 'e'
                    && BUF[i-1] != 'E'))) continue;
            if (!i)    {
                FBSetColor(0);
                FBBox(GetinpX,GetinpY,GetinpX+20*scr.W,GetinpY+scr.H);
                FBSetColor(5);
            }
            BUF[i] = c;
            sprintf(s,"%c",c);
            FBText(s,GetinpX + i*scr.W,GetinpY,0);
            i++;
        }
        if (inkey == 328 || inkey == 331) { up = -1; break; }
        if (inkey == 336 || inkey == 333) break;
    }
    BUF[i] = '\0';
    if (strlen(BUF)) *d = atof(BUF);
    FBSetColor(8);
    FBBox(GetinpX-scr.W-scr.W2,GetinpY+3,GetinpX-scr.W2,GetinpY+scr.H-3);
    if (dataok()) return (up);
    FBSetColor(0);
    FBBox(GetinpX,GetinpY,GetinpX+20*scr.W,GetinpY+scr.H);
    return (0);
}


Bool
dataok()

/* check if data makes sense */
{
    if ((Vals.ix <= 0) || (Vals.i2 <= 0))
        return (false);
    if (flags.threej && (Vals.iy <= 0))
        return (false);
    if ((Vals.la < 0) || (Vals.lb < 0) || (Vals.lc < 0) || (Vals.ld < 0))
        return (false);
    if ((Vals.la == 0) && (Vals.lb == 0))
        return (false);
    if (flags.threej && ((Vals.lc == 0) && (Vals.ld == 0)))
        return (false);
    if ((Vals.ix + Vals.i2)*(Vals.la + Vals.lb) >= 100)
        return (false);
    if ((Vals.ix + Vals.i2)*(Vals.la + Vals.lb) <= .01)
        return (false);
    if (flags.threej && ((Vals.iy + Vals.i2)*(Vals.lc + Vals.ld) >= 100))
        return (false);
    if (flags.threej && ((Vals.iy + Vals.i2)*(Vals.lc + Vals.ld) <=.01))
        return (false);
    if (Vals.lfrac < 0 || Vals.lfrac > 1)
        return (False);

    return (true);
}


Bool
getfn()

/* get fileneme, extract points, plot */
{
    FILE *fp;
    char *s;

    FBSetColor(0);
    FBBox(scr.XC-10*scr.W,scr.YL-3*scr.H,scr.XS,scr.YL-2*scr.H);
    FBSetColor(1);
    FBText("file?  : ",scr.XC-10*scr.W,scr.YL-3*scr.H,0);
    FBSetColor(4);
    FBBox(scr.XC,scr.YL-3*scr.H,scr.XU,scr.YL-2*scr.H);
    s = KbEdit(NULL,scr.XC,scr.YL-3*scr.H,4,1,2);
    if (s == NULL || !*s)
        return (true);
    strcpy(BUF,s);
    if (strlen(BUF) == 1 && *BUF == '-') {
        delpts();
        FBSetColor(0);
        FBBox(scr.XC-10*scr.W,scr.YL-3*scr.H,scr.XS,scr.YL-2*scr.H);
        return (true);
    }
    while ((fp = fopen(BUF,"r")) == NULL) {
        FBSetColor(0);
        FBBox(scr.XC-10*scr.W,scr.YL-3*scr.H,scr.XS,scr.YL-2*scr.H);
        FBSetColor(1);
        FBText("reenter : ",scr.XC-10*scr.W,scr.YL-3*scr.H,0);
        FBSetColor(4);
        FBBox(scr.XC,scr.YL-3*scr.H,scr.XU,scr.YL-2*scr.H);
        s = KbEdit(NULL,scr.XC,scr.YL-3*scr.H,4,1,2);
        if (s == NULL || !*s)
            return (true);
        strcpy(BUF,s);
    }
    FBSetColor(0);
    FBBox(scr.XC-10*scr.W,scr.YL-3*scr.H,scr.XS,scr.YL-2*scr.H);
    extpts(fp);
    plpts();
    return (false);
}


void
extpts(fp)

/* extract points from file */
FILE *fp;
{
    int i, j, x, y;
    double v1, v2;
    struct extra *p, *q;

    i = 0;
    while (fgets(BUF,80,fp)) {
        if (strchr(BUF,'p') || strchr(BUF,'f')) i++;
    }
    p    = (struct extra *) malloc(sizeof(struct extra));
    p->x = (float *) malloc(i*sizeof(float));
    p->y = (float *) malloc(i*sizeof(float));
    p->pf = (int *)  malloc(i*sizeof(int));
    p->size = i;
    p->next = NULL;
    rewind(fp);
    i = 0;
    while (fgets(BUF,80,fp)) {
        if (*BUF != '\t') sscanf(BUF,"%*d %*d %le %le",&v1,&v2);
        else if (strchr(BUF,'p')) {
            (p->x)[i] = (float) v1;
            (p->y)[i] = (float) v2;
            (p->pf)[i] = 1;
            i++;
        }
        else if (strchr(BUF,'f')) {
            (p->x)[i] = (float) v1;
            (p->y)[i] = (float) v2;
            (p->pf)[i] = 0;
            i++;
        }
    }
    if (!pts) pts = p;
    else {
        for (q = pts; q->next; q = q->next)
            ;
        q->next = p;
    }
    fclose(fp);
}


void
plpts()
/* plot points extracted from files */
{
    int i = 5, j, x, y;
    struct extra *p;

    for (p = pts; p; p = p->next) {
        FBSetColor(i++);
        for (j = 0; j < p->size; j++) {
            if (flags.f && p->pf[j]) continue;
            if (!flags.f && !p->pf[j]) continue;

            x = (p->x[j]*COUPLING - scale.xmin)*scr.X/(scale.xmax - scale.xmin)
                + scr.XL;
            if (x < scr.XL || x > scr.XU)
                return;
            y = (p->y[j] - scale.ymin)*scr.Y/(scale.ymax - scale.xmin)
                + scr.YL;
            if (y < scr.YL || y > scr.YU)
                return;
            big_pixel(x,y);

            /*
            x = (int) (.5 + ((p->x)[j] * scr.X)/
                (50*(scale.xmax-scale.xmin))) + scr.XL;
            if (flags.x) x += scr.X/2;
            y = (int) (.5 + ((p->y)[j] * scr.Y)/
                (50*(scale.ymax-scale.ymin))) + scr.YL;
            if (flags.y) y += scr.Y/2;
            if (x <= scr.XU) big_pixel(x,y);
            */
        }
    }
}


void
big_pixel(x,y)

int x, y;
{
    FBPixel(x,y+1);
    FBPixel(x-1,y);
    FBPixel(x,y);
    FBPixel(x+1,y);
    FBPixel(x,y-1);
}


void
delpts()

/* delete the last set of extracted points */
{
    struct extra *p;

    if (!pts) return;
    for (p = pts; p->next && p->next->next; p = p->next)
        ;
    if (p->next) {        
        free(p->next->x);
        free(p->next->y);
        free(p->next->pf);
        free(p->next);
        p->next = NULL;
    }
    else {
        free(p->x);
        free(p->y);
        free(p->pf);
        free(p);
        pts = NULL;
    }
}


void
xy_pick()
{
    int x,y,button;
    char key;

    flags.mark = true;
    FBSetColor(0);
    FBBox(scr.XU-38*scr.W,1,scr.XS,scr.YL-1);
    FBSetGhost(marker,0,0);
    flags.rel_x = -1;
    flags.rel_y = -1;
    for (;;) {
        FBPoint(&x,&y,&key,&button);
        switch (button) {
            case 1:
                if (x < scr.XL || x > scr.XU) continue;
                if (y < scr.YL || y > scr.YU) continue;
                refmark();
                flags.rel_x = (double)(x-scr.XL)/scr.X;
                flags.rel_y = (double)(y-scr.YL)/scr.Y;
                refmark();
                continue;
            case 4:
            case 8:
                refmark();
                flags.rel_x = -1;
                flags.rel_y = -1;
                continue;
        }
        break;
    }
    refmark();
    FBSetGhost(NULL,0,0);
    flags.mark = false;
    flags.rel_x = -1;
    flags.rel_y = -1;
}


void
refmark()

{
    int x,y;
    int z = scr.W;

    if (flags.rel_x < 0 || flags.rel_y < 0)
        return;
    x = scr.X*flags.rel_x + scr.XL + .5;
    y = scr.Y*flags.rel_y + scr.YL + .5;
    FBSetALUMode(MFBALUEOR);
    FBSetColor(1);
    FBLine(x,MFBmax(scr.YL,y-z),x,MFBmin(scr.YU,y+z));
    FBLine(MFBmax(scr.XL,x-z),y,MFBmin(scr.XU,x+z),y);
    FBSetALUMode(MFBALUJAM);
    FBUpdate();
}


void
marker(x,y,refx,refy)

int x,y,refx,refy;
{
    double x1,y1;
    int px, py;

#ifdef XWINDOWS
    y = TRAN(y);
#endif

    if (DoingRedraw) return;
    if (x < scr.XL || x > scr.XU) return;
    if (y < scr.YL || y > scr.YU) return;
    FBSetColor(1);
    FBLine(x,scr.YL,x,scr.YU);
    FBLine(scr.XL,y,scr.XU,y);

    if ((flags.rel_x < 0) || (flags.rel_y < 0)) {
        if (flags.z) {
            x1 = -PI + 2*PI*(x-scr.XL)/scr.X;
            y1 = -PI + 2*PI*(y-scr.YL)/scr.Y;
            sprintf(BUF,"x = %s, y = %s", ecvt12(x1),ecvt12(y1));
        }
        else {
            x1 = scale.xmin + (scale.xmax - scale.xmin)*(x-scr.XL)/scr.X;
            y1 = scale.ymin + (scale.ymax - scale.ymin)*(y-scr.YL)/scr.Y;
            sprintf(BUF,"Ic = %s, Ig = %s",
                ecvt12(x1/COUPLING),ecvt12(y1));
        }
    }
    else {
        px = scr.X*flags.rel_x + scr.XL + .5;
        py = scr.Y*flags.rel_y + scr.YL + .5;
        if (flags.z) {
            x1 = 2*PI*(x-px)/scr.X;
            y1 = 2*PI*(y-py)/scr.Y;
            sprintf(BUF,"dx = %s, dy = %s", ecvt12(x1),ecvt12(y1));
        }
        else {
            x1 = scale.xmin + (scale.xmax - scale.xmin)*(x-px)/scr.X;
            y1 = scale.ymin + (scale.ymax - scale.ymin)*(y-py)/scr.Y;
            sprintf(BUF,"dIc = %s, dIg = %s",
                ecvt12(x1/COUPLING),ecvt12(y1));
        }
    }
    FBText(BUF,scr.XU-strlen(BUF)*scr.W,scr.YL-scr.H3,0);
}


Bool
diag()

{
    Redraw = RD_INPT;
    if (flags.threej)
        return (diag3());
    return (diag2());
}


Bool
diag2()

/* draw diagram on screen - 2 junctions */
{
    char buf[64];
    int y;

    clear_screen();
    FBSetColor(4);
    FBLine(30,scr.H,scr.XU,scr.H);
    FBLine(scr.XU,scr.H,scr.XU,10*scr.H);
    FBLine(scr.XU,10*scr.H,30,10*scr.H);
    FBLine(30,10*scr.H,30,scr.H);

    FBSetColor(1);
    FBText("Present Values:",scr.XL,10*scr.H+scr.H2,0);
    FBText("Enter New Values (pH and mA)",scr.XC,10*scr.H+scr.H2,0);

    y = 10*scr.H + scr.H3 - 190;
        if (y < 0)
            y = 0;

    FBLine(50,y+350,60,y+340);
    FBLine(60,y+340,175,y+340);
    show_l_icon(175,y+317);
    show_x_icon(225,y+340);
    show_l_icon(275,y+317);
    FBLine(325,y+340,450,y+340);
    FBLine(450,y+340,440,y+350);
    FBLine(50,y+330,60,y+340);
    FBLine(450,y+340,440,y+330);

    FBLine(240,y+400,250,y+390);
    FBLine(260,y+400,250,y+390);
    FBLine(250,y+390,250,y+300);
    FBLine(150,y+300,175,y+300);
    show_l_icon(175,y+277);
    FBLine(225,y+300,275,y+300);
    show_l_icon(275,y+277);
    FBLine(325,y+300,350,y+300);
    FBLine(350,y+300,350,y+240);
    FBLine(350,y+240,150,y+240);
    FBLine(150,y+240,150,y+300);

    FBLine(140,y+260,160,y+280);
    FBLine(140,y+280,160,y+260);
    FBLine(340,y+260,360,y+280);
    FBLine(340,y+280,360,y+260);

    FBLine(250,y+240,250,y+210);
    FBLine(230,y+210,270,y+210);

    FBText("Ic",60,y+320,0);
    FBText("I",270,y+390,0);
    FBText("g",279,y+387,0);
    FBText("Ic1",170,y+250,0);
    FBText("Ic2",310,y+250,0);
    FBText("L1",210,y+255,0);
    FBText("L2",270,y+255,0);
    sprintf(buf, "Coupling: %f", COUPLING);
    FBText(buf, 480, y+420-scr.H,0);

    FBSetColor(3);
    FBLine(30,y+190,30,y+420);
    FBLine(30,y+420,470,y+420);
    FBLine(470,y+420,470,y+190);
    FBLine(470,y+190,30,y+190);
    FBSetColor(5);
    FBText("TWO JUNCTION INTERFEROMETER THRESHOLD CURVE PLOTTER",
        30,y+438,0);
    show_s_icon(500,y+300);

    return (entr2());
}


Bool
diag3()

/* draw diagram on screen - 3 junctions */
{
    char buf[64];
    int y;

    clear_screen();
    FBSetColor(4);
    FBLine(20,scr.H2,scr.XU,scr.H2);
    FBLine(scr.XU,scr.H2,scr.XU,10*scr.H);
    FBLine(scr.XU,10*scr.H,20,10*scr.H);
    FBLine(20,10*scr.H,20,scr.H2);

    FBSetColor(1);
    FBText("Present Values:",scr.XL,10*scr.H+scr.H2,0);
    FBText("Enter New Values (pH and mA)",scr.XC,10*scr.H+scr.H2,0);

    y = 10*scr.H + scr.H3 - 180;
        if (y < 0)
            y = 0;

    FBLine(50,y+350,60,y+340);
    FBLine(60,y+340,125,y+340);
    show_l_icon(125,y+317);
    show_x_icon(175,y+340);
    show_l_icon(225,y+317);
    FBLine(275,y+340,325,y+340);
    show_l_icon(325,y+317);
    show_x_icon(375,y+340);
    show_l_icon(425,y+317);
    FBLine(475,y+340,550,y+340);

    FBLine(550,y+340,540,y+350);
    FBLine(50,y+330,60,y+340);
    FBLine(550,y+340,540,y+330);
    FBLine(290,y+400,300,y+390);
    FBLine(310,y+400,300,y+390);

    FBLine(300,y+390,300,y+370);
    FBLine(200,y+300,200,y+370);
    FBLine(200,y+370,400,y+370);
    FBLine(400,y+370,400,y+300);

    FBLine(100,y+300,125,y+300);
    show_l_icon(125,y+277);
    FBLine(175,y+300,225,y+300);
    show_l_icon(225,y+277);
    FBLine(275,y+300,325,y+300);
    show_l_icon(325,y+277);
    FBLine(375,y+300,425,y+300);
    show_l_icon(425,y+277);
    FBLine(475,y+300,500,y+300);

    FBLine(500,y+300,500,y+240);
    FBLine(500,y+240,100,y+240);
    FBLine(100,y+240,100,y+300);
    FBLine(300,y+240,300,y+300);

    FBLine(90,y+260,110,y+280);
    FBLine(90,y+280,110,y+260);
    FBLine(290,y+260,310,y+280);
    FBLine(290,y+280,310,y+260);
    FBLine(490,y+260,510,y+280);
    FBLine(490,y+280,510,y+260);

    FBLine(300,y+240,300,y+210);
    FBLine(280,y+210,320,y+210);

    FBText("Ic",60,y+320,0);
    FBText("I",270,y+390,0);
    FBText("g",279,y+387,0);
    FBText("Ileft",180,y+373,0);
    FBText("Ic1",120,y+250,0);
    FBText("Ic2",260,y+250,0);
    FBText("L1",160,y+255,0);
    FBText("L2",220,y+255,0);
    FBText("L3",360,y+255,0);
    FBText("L4",420,y+255,0);
    FBText("Ic3",456,y+250,0);
    sprintf(buf, "Coupling: %f", COUPLING);
    FBText(buf, 590, y+430-scr.H,0);
    show_s_icon(525,y+397);

    FBSetColor(3);
    FBLine(20,y+180,20,y+430);
    FBLine(20,y+430,580,y+430);
    FBLine(580,y+430,580,y+180);
    FBLine(580,y+180,20,y+180);
    FBSetColor(5);
    FBText("THREE JUNCTION INTERFEROMETER THRESHOLD CURVE PLOTTER",20,y+450,0);

    return (entr3());
}


/* cursor height - 1 */
#define CURHT 0

/* What a pain - have to be able to redraw during editing
 * when a resize event occurs.
 */
static int doing_edit;
static int editX, editY, editFg, editCc;
static char editBuf[128], *editC;


char *
KbEdit(s,x,y,bg,fg,cc)

/*
 * s      Initial string to edit.
 * x,y    Lower left coordinates.
 * bg,fg  Background and foreground colors.
 * cc     Background color at cursor location.
 * Returns edited string (static!).
 */
char *s;
int x, y, bg, fg, cc;
{
    char tbuf[128];
    int i, k;
    char *end, ctmp[2];
    char *strchr();

    doing_edit = true;
    editFg = fg;
    editCc = cc;
    editX = x;
    editY = y;

    FBSetColor(fg);
    *editBuf = '\0';
    if (s) {
        /* s can be NULL */
        FBText(s, editX, editY, 0);
        strcpy(editBuf,s);
    }
    ctmp[1] = '\0';
    editC = editBuf;
    end = strchr(editBuf,'\0');
    *(end+1) = '\0';
    ctmp[0] = *editC;
    FBSetColor(cc);
    textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
    FBSetColor(fg);
    FBText(ctmp, editX + (int)(editC-editBuf)*scr.W, editY, 0);
    for (;;) {
        k = MFBGetchar();
        if ((char) k == '\r') break;
        switch (k) {

        case 333:    /* right arrow */
            if (editC >= end) continue;
            ctmp[0] = *editC;
            FBSetColor(bg);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            editC++;
            ctmp[0] = *editC;
            FBSetColor(cc);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            continue;

        case 331:    /* left arrow */
            if (editC <= editBuf) continue;
            ctmp[0] = *editC;
            FBSetColor(bg);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            editC--;
            ctmp[0] = *editC;
            FBSetColor(cc);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            continue;

        case '\b':
            if (editC == editBuf) continue;
            FBSetColor(bg);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            editC--;
            FBText(editC,editX + (int)(editC-editBuf)*scr.W,editY,0);
            FBSetColor(fg);
            *editC = '\0';
            strcat(editBuf,++editC);
            FBText(editC,editX + (int)(editC-editBuf)*scr.W,editY,0);
            editC--;
            if (end > editBuf) end--;
            ctmp[0] = *editC;
            FBSetColor(cc);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            continue;

        case 339: /* DEL, in DOS */
        case 127:
            if (editC == end) continue;
            FBSetColor(bg);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBText(editC,editX + (int)(editC-editBuf)*scr.W,editY,0);
            FBSetColor(fg);
            *editC = '\0';
            strcat(editBuf,++editC);
            FBText(editC,editX + (int)(editC-editBuf)*scr.W,editY,0);
            editC--;
            if (end > editBuf) end--;
            ctmp[0] = *editC;
            FBSetColor(cc);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            continue;

        case '\025': /* ^U */
        case '\030': /* ^X */
        case '\033': /* ESC */
            FBSetColor(bg);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBText(editBuf,editX,editY,0);
            editC = end = editBuf;
            *editC = '\0';
            if ((char) k == '\033') {
                doing_edit = false;
                return (NULL);
            }
            FBSetColor(cc);
            textcursor(editX,editY);
            FBSetColor(fg);
            continue;
        default:
            if (k > 255) continue;
            if (((char) k < ' ') || ((char) k > '~'))  continue;
            FBSetColor(bg);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBText(editC,editX + (int)(editC-editBuf)*scr.W,editY,0);
            strcpy(tbuf,editC);
            *editC = (char) k;
            sprintf(++editC,tbuf);
            ctmp[0] = *editC;
            FBSetColor(fg);
            FBText(editC-1,
                editX+(int)(editC-1-editBuf)*scr.W,editY,0);
            FBSetColor(cc);
            textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
            FBSetColor(fg);
            FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
            end++;
            *(end+1) = '\0';

        }
    }
    FBSetColor(bg);
    textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
    FBSetColor(fg);
    ctmp[0] = *editC;
    FBText(ctmp,editX + (int)(editC-editBuf)*scr.W,editY,0);
    doing_edit = false;
    return (editBuf);
}


void
KbRepaint(x,y)

int x,y;
{
    if (!doing_edit) return;
    editX = x;
    editY = y;
    FBSetColor(editCc);
    textcursor(editX+(int)(editC-editBuf)*scr.W,editY);
    FBSetColor(editFg);
    FBText(editBuf,editX,editY,0);
}


void
textcursor(x,y)

int x,y;
{
    FBBox(x,y-1,x+scr.W,y+CURHT);
}


void
clear_screen()

/* blank the screen */
{
    FBSetColor(0);
    FBFlood();
}

char l_icon [24][50] = {
    "00000000000010000000000001000000000000100000000000",
    "00000000000111000000000011100000000001110000000000",
    "00000000001000100000000100010000000010001000000000",
    "00000000001000100000000100010000000010001000000000",
    "00000000010000010000001000001000000100000100000000",
    "00000000010000010000001000001000000100000100000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000100000001000010000000100001000000010000000",
    "00000000010000010000001000001000000100000100000000",
    "00000000010000010000001000001000000100000100000000",
    "00000000001000100000000100010000000010001000000000",
    "00000000001000100000000100010000000010001000000000",
    "00000000000111000000000011100000000001110000000000",
    "00000000000010000000000001000000000000100000000000",
    "00000000001101100000000110110000000011011000000000",
    "00000000110000011000011000001100001100000110000000",
    "11111111000000000111100000000011110000000001111111",
};

char x_icon [7][50] = {
    "11111111111111111111000000000001111111111111111111",
    "00000000000000000001000000000001000000000000000000",
    "00000000000000000001000000000001000000000000000000",
    "00000000000000000001100000000011000000000000000000",
    "00000000000000000000100000000010000000000000000000",
    "00000000000000000000111000001110000000000000000000",
    "00000000000000000000001111111000000000000000000000",
};

char s_icon [27][50] = {
    "99999999999999999999999999999999999999999999999999",
    "00000000000000000000000000000000000000000000000000",
    "00000000000000000000000000000000000000000000000000",
    "00000000000000000000000000000000000000000000000000",
    "00;000;;000000;;;;00000000;;;;00;;;00000000;;;0000",
    "00;;;;;;;;;0000;;;00000000;;;0000;;00000000;;00000",
    "00;;;00000;;0000;;00000000;;00000;0;000000;0;00000",
    "00;;0000000;;000;;00000000;000000;0;;0000;;0;00000",
    "00;000000000;000;;0000000;;000000;00;0000;00;00000",
    "000000000000;000;;0000000;000000;;00;0000;00;;0000",
    "000000000000;000;;000000;;000000;;00;;00;;00;;0000",
    "000000000000;000;;00000;;0000000;;000;00;000;;0000",
    "000000000000;000;;00000;;0000000;0000;0;00000;0000",
    "00000000000;;000;;00000;00000000;0000;;;00000;0000",
    "0000000000;;0000;;00;;;;0000000;;0000;;000000;;000",
    "000000;;;;;00000;;;;;;;;;000000;;0000;;000000;;000",
    "000;;;;;00000000;;;00000;;00000;;000000000000;;000",
    "00;;000000000000;;0000000;;0000;;000000000000;;000",
    "00;0000000000000;;00000000;0000;00000000000000;000",
    "00;0000000000000;;00000000;0000;00000000000000;000",
    "00;0000000000000;;00000000;000;;00000000000000;;00",
    "00;0000000000000;;00000000;000;;00000000000000;;00",
    "00;000000000;000;;00000000;000;;00000000000000;;00",
    "00;;0000000;;000;;0000000;;000;;00000000000000;;00",
    "000;;00000;;;000;;;00000;;000;;;0000000000000;;;00",
    "0000;;;;;;;;;00;;;;;;;;;;000;;;;000000000000;;;;00",
    "000000;;;000;000000;;;;0000;;;;;;0000000000;;;;;;0",
};


void
show_l_icon(x,y)

/* show inductor symbol */
int x, y;
{
    int i, j;

    for (i = 0; i < 24; i++)
        for (j = 0; j < 50; j++)
            if (l_icon[i][j] - '0') FBPixel(x+j,y+i);
}


void
show_x_icon(x,y)

/* show crossover symbol */
int x, y;
{
    int i, j;

    for (i = 0; i < 7; i++)
        for (j = 0; j < 50; j++)
            if (x_icon[i][j] - '0') FBPixel(x+j,y+i);
}


void
show_s_icon(x,y)

/* show SRW icon */
int x, y;
{
    int i, j, k;

    for (i = 0; i < 27; i++)
        for (j = 0; j < 50; j++)
            if ((k = s_icon[i][j] - '0') != 0) {
                FBSetColor(k);
                FBPixel(x+j,y+i);
            }
}


void
printscreen(showdiag)

int showdiag;
{
    char *tmpDsp;
    char *outname = "ipat_plot.ps";
    char buf[128];
    int fwidth, fheight;
    int offset;

    FBSetColor(1);
    FBText("Printing...",scr.XL,scr.YL-2*scr.H,0);
    FB.fInitialized = False;
    FB.fInterface = 1;
    tmpDsp = FB.fDisplay;
    FBBegin(outname);
    fheight = FBInfo(FONTHEIGHT);
    fwidth  = FBInfo(FONTWIDTH);
    offset  = 3*FBInfo(MAXY)/8;
    scr.W   = fwidth;
    scr.H   = fheight;
    scr.W2  = fwidth/2;
    scr.H2  = fheight/2;
    scr.H3  = scr.H + scr.H2;
    scr.XL  = 4*fwidth;
    scr.YL  = offset + 3*fheight;
    scr.XS  = FBInfo(MAXX);
    scr.YS  = offset + FBInfo(MAXY)/2;
    scr.X   = scr.XS - scr.XL - 2*fwidth;
    scr.Y   = scr.YS - scr.YL - 3*fheight;
    scr.XC  = scr.XL + scr.X/2;
    scr.YC  = scr.YL + scr.Y/2;
    scr.XU  = scr.XL + scr.X;
    scr.YU  = scr.YL + scr.Y;
    DoingPrint = true;
    diag();
    displ();
    DoingPrint = false;
    FBText("IPAT utility v2.3",scr.XL,scr.YU + 5*scr.H, 0);
    FBEnd();
    FB.fDisplay = tmpDsp;
    FB.fInterface = 0;
    fheight = FBInfo(FONTHEIGHT);
    fwidth  = FBInfo(FONTWIDTH);
    scr.W   = fwidth;
    scr.H   = fheight;
    scr.W2  = fwidth/2;
    scr.H2  = fheight/2;
    scr.H3  = scr.H + scr.H2;
    scr.XL  = 4*fwidth;
    scr.YL  = 3*fheight;
    scr.XS  = FBInfo(MAXX);
    scr.YS  = FBInfo(MAXY);
    scr.X   = scr.XS - scr.XL - 2*fwidth;
    scr.Y   = scr.YS - scr.YL - 3*fheight;
    scr.XC  = scr.XL + scr.X/2;
    scr.YC  = scr.YL + scr.Y/2;
    scr.XU  = scr.XL + scr.X;
    scr.YU  = scr.YL + scr.Y;
    if (PRINTCMD && *PRINTCMD) {
        sprintf(buf, "%s %s", PRINTCMD, outname);
printf("%s\n", buf);
        system(buf);
    }
    if (showdiag) {
        DoingPrint = true;
        DoingRedraw = true;
        diag();
        DoingRedraw = false;
        DoingPrint = false;
    }
}


char *
ecvt12(d)

/*
 * Format: space or +, x.xxxxx, e, + or -, xx
 * keeps track of 10 calls for use in printf.
 */
double d;
{
    static char s[10][16];
    static int num;
    int xp;
    char *ss;

    num++;
    if (num > 9) num = 0;
    sprintf(s[num],"%+.5e",d);
    ss = strchr(s[num],'e');
    ss++;
    xp = atoi(ss);

    if (xp > 99) xp = 99;
    if (xp < -99) {
        sprintf(s[num]," 0.00000e+00");
        return s[num];
    }

    *(s[num]+8) = 'e';
    if (xp < 0)    {
        *(s[num]+9) = '-';
        xp = -xp;
    }
    else
        *(s[num]+9) = '+';
    *(s[num]+10) = '0' + (xp/10);
    *(s[num]+11) = '0' + (xp%10);
    *(s[num]+12) = '\0';
    if (*s[num] == '+')
        *s[num] = ' ';
    return s[num];
}

#ifdef XWINDOWS

static int reds[] =
    { 0, 60, 60, 20, 35, 60, 20, 60, 50, 45,  0,  0, 50, 25, 50, 60 } ;
static int grns[] =
    { 0, 60,  0, 60, 25, 60, 60, 30, 50, 45, 50,  0, 30, 35,  0, 45 } ;
static int blus[] =
    { 0, 60,  0, 20, 60, 30, 60, 60, 50, 15,  0, 40, 20, 40, 50, 15 } ;

void
SetColors()
/* set the colors */
{
    int i;
    int red, green, blue;

    for (i = 1; i < 16; i++) {
        red = reds[i] << 4;
        green = grns[i] << 4;
        blue = blus[i] << 4;
        FBDefineColor(i, red, green, blue);
    }
}

void
RepaintWindow(code)
int code;
{
    if (code) return;
    DoingRedraw = 1;
    grafinit();
    switch (Redraw) {
        case RD_INPT:
            diag();
            break;
        case RD_DISP:
            displ();
            if (flags.mark) {
                FBSetColor(0);
                FBBox(scr.XC-8*scr.W,1,scr.XS,scr.YL-1);
                refmark();
            }
            break;
        case RD_HELP:
            showhelp();
            break;
    }
    DoingRedraw = 0;
}

makehpplot() {return 0;}

/* Routines to communicate with the X MFB package to set the
 * window positions on screen.
 */
 
static struct {
    int X;
    int Y;
    int W;
    int H;
} WHERE;
 
void
SetWhereDisplayWindow(x,y,w,h)
/* Called from application. x and y are upper left position, w and h
 * are width and height.  Each is normalized to 0-100, 100 for
 * full screen window.
 */
int x,y,w,h;
{
    WHERE.X = x;
    WHERE.Y = y;
    WHERE.W = w;
    WHERE.H = h;
}
 
void
SetDisplayWindow(sw,sh,x,y,w,h)
/* Called from window manager. sw and sh are actual pixel width, height.
 * x,y,w,h are pointers to actual pixel values.
 */
int sw,sh,*x,*y,*w,*h;
{
    *x = (WHERE.X*sw)/100;
    *y = (WHERE.Y*sh)/100;
    *w = (WHERE.W*sw)/100;
    *h = (WHERE.H*sh)/100;
}
 
#else

void
SetWhereDisplayWindow() {}

void
SetColors() {}

#endif
