/*************************************************************************
 MFB graphics and miscellaneous library
 Copyright (c) Stephen R. Whiteley 1992
 Author: Stephen R. Whiteley
 *************************************************************************/

#include "mfb.h"
#if defined(MSDOS) || __MSDOS__
#include "mfbP.h"
#endif

/* A New polygon clipping routine.  Unlike the old routine, this one
 * keeps track of severed polygons by means of a linked list.
 * First call MFBNewPolygonClip(), which will perform clipping and
 * create a list of polygons.  The polygons are then extracted with
 * MFBNewPolygon() until NULL is returned.  All storage is static.  A
 * suitably large xy buffer must be supplied in the MFBPOLYGON
 * structure supplied to MFBNewPolygon().
 */


/* We represent individual polygons with a circular doubly linked list */
struct pair {
    long pa_x,pa_y;
    struct pair *pa_next;
    struct pair *pa_prev;
};

/* List of circularly linked polygons */
struct plist {
    struct pair *pl_pair;
    struct plist *pl_next;
};

/* object which will be sorted */
struct vertx {
    struct pair *v_pa;
    struct plist *v_pl;
    char v_type;
};


static struct plist *Polylist;           /* head of polygon list       */
static struct plist *Newpoly;            /* free structure to allocate */
static struct plist *Polycurrent;        /* current polygon to return  */
static struct plist Polylistbuf[100];    /* polygon list storage       */
static struct vertx Tmplst[100];         /* temp storage for sorting   */
static struct pair  *Newlink;            /* free structure to allocate */
static struct pair  Linkbuf[400];        /* polygon point storage      */


#ifdef __STDC__
static void pclip_right(long);
static void pclip_left(long);
static void pclip_top(long);
static void pclip_bottom(long);
static void linkpolys(struct plist**,int);
static struct vertx *sortlist(struct plist**,int);
static void sort(struct vertx*,int,int,int);
#else
static void pclip_right();
static void pclip_left();
static void pclip_top();
static void pclip_bottom();
static void linkpolys();
static struct vertx *sortlist();
static void sort();
#endif


void
MFBNewPolygonClip(poly,left,bottom,right,top)

MFBPOLYGON *poly;
long top,bottom,left,right;
{
    struct plist *pl;
    struct pair *pa;
    int i, n;

    Newlink = Linkbuf;
    pa = Newlink;
    Newpoly = Polylistbuf;
    Polylist = Newpoly++;
    n = (poly->nvertices << 1) - 2;
    for (i = 0; i < n; i += 2, Newlink++) {
        Newlink->pa_x = poly->xy[i];
        Newlink->pa_y = poly->xy[i+1];
        Newlink->pa_next  = Newlink+1;
        Newlink->pa_prev = Newlink-1;
    }
    if (pa->pa_x != poly->xy[i] || pa->pa_y != poly->xy[i+1]) {
        Newlink->pa_x = poly->xy[i];
        Newlink->pa_y = poly->xy[i+1];
        Newlink->pa_next  = Newlink+1;
        Newlink->pa_prev = Newlink-1;
        Newlink++;
    }
    Newlink--;
    Newlink->pa_next = pa;
    pa->pa_prev = Newlink;
    Newlink++;
    Polylist->pl_next = NULL;
    Polylist->pl_pair = pa;

    pclip_right(right);
    pclip_left(left);
    pclip_top(top);
    pclip_bottom(bottom);

    Polycurrent = Polylist;
}


int
MFBNewPolygon(p)

MFBPOLYGON *p;
{
    long *xy;
    int i;
    struct pair *pp, *pa;

again:
    xy = (long*)p->xy;
    i = 1;
    if (Polycurrent == NULL) return 0;
    pp = pa = Polycurrent->pl_pair;
    *xy++ = pp->pa_x;
    *xy++ = pp->pa_y;

    for (pp = pp->pa_next; ; pp = pp->pa_next) {
        i++;
        *xy++ = pp->pa_x;
        *xy++ = pp->pa_y;
        if (pp == pa) break;
    }

    Polycurrent = Polycurrent->pl_next;
    if (i < 4)
        goto again;
    p->nvertices = i;
    return MFBOK;
}


static void
pclip_right(right)

long right;
{
    struct plist *p, *plast = NULL, *pltmp, *pbegin;
    struct pair *pp, *pa, *pfirst, *pprev;
    long tmp, x1, x2, y1, y2;
    int madeone;

    /* Clip along the right edge.  Process each polygon in list, linking
     * in new polygons as necessary.
     */

    for (p = Polylist; p; p = p->pl_next) {
        pa = p->pl_pair;

        /* find first inside point after outside point */
        for (pp = pa->pa_next; pp != pa; pp = pp->pa_next)
            if (pp->pa_x <= right && pp->pa_prev->pa_x > right) break;

        if (pp == pa) {
            if (pp->pa_x <= right && pp->pa_prev->pa_x <= right) {
                /* all points inside, no action needed */
                plast = p;
                continue;
            }
            if (pp->pa_x > right && pp->pa_prev->pa_x > right) {
                /* all points outside, delete from list */
                if (plast == NULL)
                    Polylist = p->pl_next;
                else
                    plast->pl_next = p->pl_next;
                continue;
            }
        }

        pa = pp;
        madeone = 0;
        do {
            /* start poly, edge point */
            if (madeone) {
                Newpoly->pl_next = p->pl_next;
                p->pl_next = Newpoly;
                p = Newpoly;
                Newpoly++;
            }
            /* compute Y intercept */
            pfirst = pp->pa_prev;
            x1 = pfirst->pa_x;
            y1 = pfirst->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = y1 + ((right - x1) * (y2 - y1))/(x2 - x1);

            if (pp->pa_x != right || pp->pa_y != tmp) {
                pfirst = Newlink++;
                pfirst->pa_x = right;
                pfirst->pa_y = tmp;
                pfirst->pa_next = pp;
                pp->pa_prev = pfirst;
            }
            else
                pfirst = pp;

            p->pl_pair = pfirst;

            /* step through interior points */
            for (; pp->pa_x <= right; pp = pp->pa_next) ;

            /* add last edge and complete */
            /* compute Y intercept */
            pprev = pp->pa_prev;
            x1 = pprev->pa_x;
            y1 = pprev->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = y1 + ((right - x1) * (y2 - y1))/(x2 - x1);

            if (pprev->pa_x == right && pprev->pa_y == tmp) {
                pprev->pa_next = pfirst;
                pfirst->pa_prev = pprev;
            }
            else {
                pprev->pa_next = Newlink;
                Newlink->pa_next = pfirst;
                Newlink->pa_prev = pprev;
                Newlink->pa_y = tmp;
                Newlink->pa_x = right;
                pfirst->pa_prev = Newlink;
                Newlink++;
            }

            madeone++;
            /* move to next interior point */
            for (; pp->pa_x > right; pp = pp->pa_next) ;
        } while (pp != pa);

        /* Now see if any of the newly generated polygons should be
         * combined.
         */

        if (madeone > 1) {
            pltmp = p->pl_next;
            p->pl_next = NULL;
            pbegin = (plast == NULL ? Polylist : plast->pl_next);
            linkpolys(&pbegin,'y');
            if (plast == NULL)
                Polylist = pbegin;
            else
                plast->pl_next = pbegin;

            for (plast = pbegin; plast->pl_next; plast = plast->pl_next) ;
            plast->pl_next = pltmp;
            p = plast;
        }
        else
            plast = p;
    }
}


static void
pclip_left(left)

long left;
{
    struct plist *p, *plast = NULL, *pltmp, *pbegin;
    struct pair *pp, *pa, *pfirst, *pprev;
    long tmp, x1, x2, y1, y2;
    int madeone;

    /* Clip along the left edge.  Process each polygon in list, linking
     * in new polygons as necessary.
     */

    for (p = Polylist; p; p = p->pl_next) {
        pa = p->pl_pair;

        /* find first inside point after outside point */
        for (pp = pa->pa_next; pp != pa; pp = pp->pa_next)
            if (pp->pa_x >= left && pp->pa_prev->pa_x < left) break;

        if (pp == pa) {
            if (pp->pa_x >= left && pp->pa_prev->pa_x >= left) {
                /* all points inside, no action needed */
                plast = p;
                continue;
            }
            if (pp->pa_x < left && pp->pa_prev->pa_x < left) {
                /* all points outside, delete from list */
                if (plast == NULL)
                    Polylist = p->pl_next;
                else
                    plast->pl_next = p->pl_next;
                continue;
            }
        }

        pa = pp;
        madeone = 0;
        do {
            /* start poly, edge point */
            if (madeone) {
                Newpoly->pl_next = p->pl_next;
                p->pl_next = Newpoly;
                p = Newpoly;
                Newpoly++;
            }
            /* compute Y intercept */
            pfirst = pp->pa_prev;
            x1 = pfirst->pa_x;
            y1 = pfirst->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = y1 + ((left - x1) * (y2 - y1))/(x2 - x1);

            if (pp->pa_x != left || pp->pa_y != tmp) {
                pfirst = Newlink++;
                pfirst->pa_x = left;
                pfirst->pa_y = tmp;
                pfirst->pa_next = pp;
                pp->pa_prev = pfirst;
            }
            else
                pfirst = pp;

            p->pl_pair = pfirst;

            /* step through interior points */
            for (; pp->pa_x >= left; pp = pp->pa_next) ;

            /* add last edge and complete */
            /* compute Y intercept */
            pprev = pp->pa_prev;
            x1 = pprev->pa_x;
            y1 = pprev->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = y1 + ((left - x1) * (y2 - y1))/(x2 - x1);

            if (pprev->pa_x == left && pprev->pa_y == tmp) {
                pprev->pa_next = pfirst;
                pfirst->pa_prev = pprev;
            }
            else {
                pprev->pa_next = Newlink;
                Newlink->pa_next = pfirst;
                Newlink->pa_prev = pprev;
                Newlink->pa_y = tmp;
                Newlink->pa_x = left;
                pfirst->pa_prev = Newlink;
                Newlink++;
            }
            madeone++;
            /* move to next interior point */
            for (; pp->pa_x < left; pp = pp->pa_next) ;
        } while (pp != pa);

        /* Now see if any of the newly generated polygons should be
         * combined.
         */

        if (madeone > 1) {
            pltmp = p->pl_next;
            p->pl_next = NULL;
            pbegin = (plast == NULL ? Polylist : plast->pl_next);
            linkpolys(&pbegin,'y');
            if (plast == NULL)
                Polylist = pbegin;
            else
                plast->pl_next = pbegin;

            for (plast = pbegin; plast->pl_next; plast = plast->pl_next) ;
            plast->pl_next = pltmp;
            p = plast;
        }
        else
            plast = p;
    }
}


static void
pclip_top(top)

long top;
{
    struct plist *p, *plast = NULL, *pltmp, *pbegin;
    struct pair *pp, *pa, *pfirst, *pprev;
    long tmp, x1, x2, y1, y2;
    int madeone;

    /* Clip along the top edge.  Process each polygon in list, linking
     * in new polygons as necessary.
     */

    for (p = Polylist; p; p = p->pl_next) {
        pa = p->pl_pair;

        /* find first inside point after outside point */
        for (pp = pa->pa_next; pp != pa; pp = pp->pa_next)
            if (pp->pa_y <= top && pp->pa_prev->pa_y > top) break;

        if (pp == pa) {
            if (pp->pa_y <= top && pp->pa_prev->pa_y <= top) {
                /* all points inside, no action needed */
                plast = p;
                continue;
            }
            if (pp->pa_y > top && pp->pa_prev->pa_y > top) {
                /* all points outside, delete from list */
                if (plast == NULL)
                    Polylist = p->pl_next;
                else
                    plast->pl_next = p->pl_next;
                continue;
            }
        }

        pa = pp;
        madeone = 0;
        do {
            /* start poly, edge point */
            if (madeone) {
                Newpoly->pl_next = p->pl_next;
                p->pl_next = Newpoly;
                p = Newpoly;
                Newpoly++;
            }
            /* compute X intercept */
            pfirst = pp->pa_prev;
            x1 = pfirst->pa_x;
            y1 = pfirst->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = x1 + ((top - y1) * (x2 - x1))/(y2 - y1);

            if (pp->pa_x != tmp || pp->pa_y != top) {
                pfirst = Newlink++;
                pfirst->pa_x = tmp;
                pfirst->pa_y = top;
                pfirst->pa_next = pp;
                pp->pa_prev = pfirst;
            }
            else
                pfirst = pp;

            p->pl_pair = pfirst;

            /* step through interior points */
            for (; pp->pa_y <= top; pp = pp->pa_next) ;

            /* add last edge and complete */
            /* compute X intercept */
            pprev = pp->pa_prev;
            x1 = pprev->pa_x;
            y1 = pprev->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = x1 + ((top - y1) * (x2 - x1))/(y2 - y1);

            if (pprev->pa_x == tmp && pprev->pa_y == top) {
                pprev->pa_next = pfirst;
                pfirst->pa_prev = pprev;
            }
            else {
                pprev->pa_next = Newlink;
                Newlink->pa_next = pfirst;
                Newlink->pa_prev = pprev;
                Newlink->pa_y = top;
                Newlink->pa_x = tmp;
                pfirst->pa_prev = Newlink;
                Newlink++;
            }
            madeone++;
            /* move to next interior point */
            for (; pp->pa_y > top; pp = pp->pa_next) ;
        } while (pp != pa);

        /* Now see if any of the newly generated polygons should be
         * combined.
         */

        if (madeone > 1) {
            pltmp = p->pl_next;
            p->pl_next = NULL;
            pbegin = (plast == NULL ? Polylist : plast->pl_next);
            linkpolys(&pbegin,'x');
            if (plast == NULL)
                Polylist = pbegin;
            else
                plast->pl_next = pbegin;

            for (plast = pbegin; plast->pl_next; plast = plast->pl_next) ;
            plast->pl_next = pltmp;
            p = plast;
        }
        else
            plast = p;
    }
}


static void
pclip_bottom(bottom)

long bottom;
{
    struct plist *p, *plast = NULL, *pltmp, *pbegin;
    struct pair *pp, *pa, *pfirst, *pprev;
    long tmp, x1, x2, y1, y2;
    int madeone;

    /* Clip along the bottom edge.  Process each polygon in list, linking
     * in new polygons as necessary.
     */

    for (p = Polylist; p; p = p->pl_next) {
        pa = p->pl_pair;

        /* find first inside point after outside point */
        for (pp = pa->pa_next; pp != pa; pp = pp->pa_next)
            if (pp->pa_y >= bottom && pp->pa_prev->pa_y < bottom) break;

        if (pp == pa) {
            if (pp->pa_y >= bottom && pp->pa_prev->pa_y >= bottom) {
                /* all points inside, no action needed */
                plast = p;
                continue;
            }
            if (pp->pa_y < bottom && pp->pa_prev->pa_y < bottom) {
                /* all points outside, delete from list */
                if (plast == NULL)
                    Polylist = p->pl_next;
                else
                    plast->pl_next = p->pl_next;
                continue;
            }
        }

        pa = pp;
        madeone = 0;
        do {
            /* start poly, edge point */
            if (madeone) {
                Newpoly->pl_next = p->pl_next;
                p->pl_next = Newpoly;
                p = Newpoly;
                Newpoly++;
            }
            /* compute X intercept */
            pfirst = pp->pa_prev;
            x1 = pfirst->pa_x;
            y1 = pfirst->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = x1 + ((bottom - y1) * (x2 - x1))/(y2 - y1);

            if (pp->pa_x != tmp || pp->pa_y != bottom) {
                pfirst = Newlink++;
                pfirst->pa_x = tmp;
                pfirst->pa_y = bottom;
                pfirst->pa_next = pp;
                pp->pa_prev = pfirst;
            }
            else
                pfirst = pp;

            p->pl_pair = pfirst;

            /* step through interior points */
            for (; pp->pa_y >= bottom; pp = pp->pa_next) ;

            /* add last edge and complete */
            /* compute X intercept */
            pprev = pp->pa_prev;
            x1 = pprev->pa_x;
            y1 = pprev->pa_y;
            x2 = pp->pa_x;
            y2 = pp->pa_y;
            tmp = x1 + ((bottom - y1) * (x2 - x1))/(y2 - y1);

            if (pprev->pa_x == tmp && pprev->pa_y == bottom) {
                pprev->pa_next = pfirst;
                pfirst->pa_prev = pprev;
            }
            else {
                pprev->pa_next = Newlink;
                Newlink->pa_next = pfirst;
                Newlink->pa_prev = pprev;
                Newlink->pa_y = bottom;
                Newlink->pa_x = tmp;
                pfirst->pa_prev = Newlink;
                Newlink++;
            }
            madeone++;
            /* move to next interior point */
            for (; pp->pa_y < bottom; pp = pp->pa_next) ;
        } while (pp != pa);

        /* Now see if any of the newly generated polygons should be
         * combined.
         */

        if (madeone > 1) {
            pltmp = p->pl_next;
            p->pl_next = NULL;
            pbegin = (plast == NULL ? Polylist : plast->pl_next);
            linkpolys(&pbegin,'x');
            if (plast == NULL)
                Polylist = pbegin;
            else
                plast->pl_next = pbegin;

            for (plast = pbegin; plast->pl_next; plast = plast->pl_next) ;
            plast->pl_next = pltmp;
            p = plast;
        }
        else
            plast = p;
    }
}


static void
linkpolys(pp,c)

struct plist **pp;
int c;
{
    struct plist *p;
    struct pair *p1,*p2;
    struct vertx *list;
    int i,j;

    /* Some of the polys found so far should be linked into single
     * polys.  This would for example give a band rather than two
     * overlapping shapes.  The logic is to first create an array
     * of vertex structures, which represent the endpoints of each
     * saved poly.  The array is sorted is ascending order.  The
     * vertices, taken two at a time, are linked, according to the
     * direction code stored with the vertex.  The linked-to poly
     * entry is replaced with the linked-from poly entry throughout
     * the list.  When all is done, the poly references that remain
     * in the list are returned.
     */

    /* sort endpoints into ascending order by intercept point */
    list = sortlist(pp,c);

    for (i = 0; list[i].v_pa != NULL; i += 2) {
        p1 = list[i].v_pa;
        p2 = list[i+1].v_pa;
        if (list[i].v_type == 'e') {
            /*  At end of p1, link on p2 (voodoo: p2 is beginning). */
            p1->pa_next = p2;
            p2->pa_prev = p1;
            p = list[i+1].v_pl;
            /* The poly containing p2 is now part of
             * the poly containing p1. */
            for (j = 0; list[j].v_pa != NULL; j++)
                if (list[j].v_pl == p)
                    list[j].v_pl = list[i].v_pl;
        }
        else {
            /* At beginning of p1, link to p2 (voodoo: p2 is end). */
            p2->pa_next = p1;
            p1->pa_prev = p2;
            /* The poly containing p1 is now part of
             * the poly containing p2. */
            p = list[i].v_pl;
            for (j = 0; list[j].v_pa != NULL; j++)
                if (list[j].v_pl == p)
                    list[j].v_pl = list[i+1].v_pl;
        }
    }

    /* Form a list of the polys that remain. */
    p = *pp = list[0].v_pl;
    for (i = 2; list[i].v_pa != NULL; i += 2) {
        if (p != list[i].v_pl) {
            p->pl_next = list[i].v_pl;
            p = p->pl_next;
        }
    }
    p->pl_next = NULL;
}


static struct vertx *
sortlist(p,c)

struct plist **p;
int c;
{
    int i;
    struct plist *q;
    struct vertx tmp;

    for (i = 0, q = *p; q; q = q->pl_next) {
        Tmplst[i].v_pa = q->pl_pair;
        Tmplst[i].v_pl = q;
        Tmplst[i].v_type = 'b';
        i++;
        Tmplst[i].v_pa = q->pl_pair->pa_prev;
        Tmplst[i].v_pl = q;
        Tmplst[i].v_type = 'e';
        i++;
    }
    Tmplst[i].v_pa = NULL;

    sort(Tmplst,0,i-1,c);

    /* This code puts degenerate vertices in proper order.  All bets are
     * off if the original poly is self-overlapping.
     */
    c = Tmplst[0].v_type;
    for (i = 2; Tmplst[i].v_pa; i += 2) {
        if (Tmplst[i].v_type != c) {
            tmp = Tmplst[i];
            Tmplst[i] = Tmplst[i-1];
            Tmplst[i-1] = tmp;
        }
    }

    return Tmplst;
}


static void
sort(lst,left,right,c)

struct vertx *lst;
int left,right;
int c;
{
    int i, last;
    struct vertx tmp;
    long e1,e2;

    if (right <= left) return;
    i = (left + right) >> 1;

    /* swap i,left */
    tmp = *(lst+i);
    *(lst+i) = *(lst+left);
    *(lst+left) = tmp;

    last = left;
    for (i = left + 1; i <= right; i++) {
        if (c == 'x') {
            e1 = lst[i].v_pa->pa_x;
            e2 = lst[left].v_pa->pa_x;
        }
        else {
            e1 = lst[i].v_pa->pa_y;
            e2 = lst[left].v_pa->pa_y;
        }
        if (e1 < e2) {
            last++;
            /* swap i,last */
            tmp = *(lst+i);
            *(lst+i) = *(lst+last);
            *(lst+last) = tmp;
        }
    }

    /* swap last,left */
    tmp = *(lst+last);
    *(lst+last) = *(lst+left);
    *(lst+left) = tmp;

    sort(lst,left,last-1,c);
    sort(lst,last+1,right,c);
}
